<?php

namespace MotoPress\Appointment;

if (!defined('ABSPATH')) {
    exit;
}

/**
 * @since 1.0
 */
class Plugin
{
    /**
     * @var self
     *
     * @since 1.0
     */
    protected static $instance = null;

    /**
     * @var Plugin\Assets
     *
     * @since 1.0
     */
    protected $assets = null;

    /**
     * @var Plugin\I18n
     *
     * @since 1.0
     */
    protected $i18n = null;

    /**
     * @var Plugin\Settings
     *
     * @since 1.0
     */
    protected $settings = null;

	/**
	 * @since 1.4.0
	 * @var Plugin\Upgrader
	 */
	protected $upgrader = null;

    /**
     * @var Emails\EmailsDispatcher
     *
     * @since 1.1.0
     */
    protected $emailsDispatcher = null;

    /**
     * @var array
     *
     * @since 1.0
     */
    protected $registries = [];

    /**
     * @since 1.0
     */
    public function setup()
    {
        // Setup only basics at this point and leave others for 'plugins_loaded'
        // and 'init' actions
        $this->i18n = new Plugin\I18n();
        $this->settings = new Plugin\Settings();
		$this->upgrader = new Plugin\Upgrader();
        $this->emailsDispatcher = new Emails\EmailsDispatcher();

        // Setup request-specific sections
        if (!wp_doing_ajax() && !wp_doing_cron()) {
            $this->assets = new Plugin\Assets();
        }

        // Setup registries
        $this->registries['bundles']      = new Registries\BundlesRegistry();
        $this->registries['emails']       = new Registries\EmailsRegistry();
        $this->registries['metaboxes']    = new Registries\MetaboxesRegistry();
        $this->registries['pages']        = new Registries\PagesRegistry();
        $this->registries['postTypes']    = new Registries\PostTypesRegistry();
        $this->registries['repositories'] = new Registries\RepositoriesRegistry();
        $this->registries['rest']         = new Registries\RestRegistry();
        $this->registries['shortcodes']   = new Registries\ShortcodesRegistry();
        $this->registries['templates']    = new Registries\TemplatesRegistry();
        $this->registries['widgets']      = new Registries\WidgetsRegistry();

        // Use priority 5/15, so addons can safely load/init on default priority
        add_action('plugins_loaded', [$this, 'load'], 15);
        add_action('init', [$this, 'init'], 15);
        add_action('rest_api_init', [$this, 'initRest'], 15);
    }

    /**
     * @access protected
     *
     * @since 1.0
     */
    public function load()
    {
        // Load translations
        $this->loadTextdomain();

        // Register global items (admin and frontend)
        $this->emailsDispatcher->load();

        $this->registries['postTypes']->registerAll();
        $this->registries['shortcodes']->registerAll();
        $this->registries['widgets']->registerAll();

        // Register admin-only items
        if (is_admin()) {
            $this->registries['pages']->registerCustomPages();
            $this->registries['metaboxes']->addListeners();
        }

        /** @since 1.0 */
        do_action('mpa_plugin_loaded', $this);
    }

    /**
     * @since 1.2.1
     */
    public function loadTextdomain()
    {
        load_plugin_textdomain('motopress-appointment', false, mpa_languages_dir());
    }

    /**
     * @access protected
     *
     * @since 1.0
     */
    public function init()
    {
        $this->initOnce();

        // Register admin-only items
        if (is_admin()) {
            $this->registries['pages']->registerManagePostsPages();
            $this->registries['pages']->registerEditPostPages();
        }

        new Elementor\Init();
        new Divi\Init();
        new Gutenberg\Init();

        /** @since 1.0 */
        do_action('mpa_plugin_inited', $this);
    }

    /**
     * @since 1.0
     */
    protected function initOnce()
    {
        if ((bool)get_option('mpa_permalinks_need_update', false)) {
            flush_rewrite_rules();

            delete_option('mpa_permalinks_need_update');
        }
    }

    /**
     * @access protected
     *
     * @since 1.0
     */
    public function initRest()
    {
        // Register REST controllers
        $this->registries['rest']->registerAll();

        /** @since 1.0 */
        do_action('mpa_rest_inited', $this);
    }

    /**
     * @return Plugin\Assets
     *
     * @since 1.0
     */
    public function assets()
    {
        return $this->assets;
    }

    /**
     * @return Emails\EmailsDispatcher
     *
     * @since 1.1.0
     */
    public function emailsDispatcher()
    {
        return $this->emailsDispatcher;
    }

    /**
     * @return Plugin\I18n
     *
     * @since 1.0
     */
    public function i18n()
    {
        return $this->i18n;
    }

    /**
     * @return Plugin\Settings
     *
     * @since 1.0
     */
    public function settings()
    {
        return $this->settings;
    }

	/**
	 * @since 1.4.0
	 *
	 * @return Plugin\Upgrader
	 */
	public function upgrader()
	{
		return $this->upgrader;
	}

    /**
     * @return Registries\BundlesRegistry
     *
     * @since 1.0
     */
    public function bundles()
    {
        return $this->registries['bundles'];
    }

    /**
     * @return Registries\EmailsRegistry
     *
     * @since 1.1.0
     */
    public function emails()
    {
        return $this->registries['emails'];
    }

    /**
     * @return Registries\MetaboxesRegistry
     *
     * @since 1.0
     */
    public function metaboxes()
    {
        return $this->registries['metaboxes'];
    }

    /**
     * @return Registries\PagesRegistry
     *
     * @since 1.0
     */
    public function pages()
    {
        return $this->registries['pages'];
    }

    /**
     * @return Registries\PostTypesRegistry
     *
     * @since 1.0
     */
    public function postTypes()
    {
        return $this->registries['postTypes'];
    }

    /**
     * @return Registries\RepositoriesRegistry
     *
     * @since 1.0
     */
    public function repositories()
    {
        return $this->registries['repositories'];
    }

    /**
     * @return Registries\RestRegistry
     *
     * @since 1.0
     */
    public function rest()
    {
        return $this->registries['rest'];
    }

    /**
     * @return Registries\ShortcodesRegistry
     *
     * @since 1.0
     */
    public function shortcodes()
    {
        return $this->registries['shortcodes'];
    }

    /**
     * @return Registries\TemplatesRegistry
     *
     * @since 1.1.0
     */
    public function templates()
    {
        return $this->registries['templates'];
    }

    /**
     * @return Registries\WidgetsRegistry
     *
     * @since 1.3
     */
    public function widgets()
    {
        return $this->registries['widgets'];
    }

    /**
     * @access protected
     *
     * @since 1.0
     */
    public function activate()
    {
        // Create dir /uploads/motopress-appointment/
        Services\Installer::createUploadDir();

        // Flush rewrite rules on next init
        add_option('mpa_permalinks_need_update', true);

		// Set up the initial DB version
		if (is_null(get_option('mpa_db_version', null))) {
			update_option('mpa_db_version', mpa_version());
		}
    }

    /**
     * @return self
     *
     * @since 1.0
     */
    public static function getInstance()
    {
        if (is_null(static::$instance)) {
            static::$instance = new static();
        }

        return static::$instance;
    }
}
