<?php

namespace MotoPress\Appointment\AdminPages\Custom;

if (!defined('ABSPATH')) {
    exit;
}

/**
 * @since 1.0
 */
class SettingsPage extends AbstractCustomPage
{
    /**
     * Don't access directly. This is a cache field. Use getCurrentTab()
     * instead.
     *
     * @var string|null
     *
     * @see SettingsPage::getCurrentTab()
     *
     * @since 1.0
     */
    protected $currentTab = null;

    /**
     * Don't access directly. This is a cache field. Use getCurrentSection()
     * instead.
     *
     * @var string|null
     *
     * @see SettingsPage::getCurrentSection()
     *
     * @since 1.1.0
     */
    protected $currentSection = null;

    /**
     * @var \MotoPress\Appointment\Fields\AbstractField[] [Option name => Field object]
     *
     * @since 1.0
     */
    protected $fields = [];

    /**
     * @var bool
     *
     * @since 1.0
     */
    protected $saved = false;

    /**
     * @access protected
     *
     * @since 1.1.0
     */
    public function enqueueScripts()
    {
        if ($this->getCurrentTab() == 'email') {
            mpapp()->assets()->enqueueBundle('mpa-spectrum');
            mpapp()->assets()->enqueueBundle('mpa-settings-page');
        }
    }

    /**
     * @access protected
     *
     * @since 1.0
     */
    public function load()
    {
        // Create fields
        $currentTab = $this->getCurrentTab();
        $currentSection = $this->getCurrentSection();

        $fields = $this->getFields($currentTab, $currentSection);
        $fields = mpa_create_fields($fields);

        $this->fields += $fields;

        // Maybe save changes
        $this->maybeSave();
    }

    /**
     * @since 1.0
     */
    protected function maybeSave()
    {
        if (!$this->canSave()) {
            return;
        }

        // Parse request
        $newValues = $this->parseRequest();

        foreach ($newValues as $optionName => $optionValue) {
            update_option($optionName, $optionValue, 'yes');
        }

        $this->saved = true;
    }

    /**
     * @return bool
     *
     * @since 1.0
     */
    protected function canSave()
    {
        if (!isset($_POST['mpa_save_settings'])) {
            return false;
        }

        if (!current_user_can('manage_options')) {
            return false;
        }

        $currentTab = $this->getCurrentTab();

        $nonceAction = "mpa_save_{$currentTab}_settings";
        $nonceField = "_mpa_{$currentTab}_settings_nonce";

        if (!mpa_verify_nonce($nonceAction, $nonceField)) {
            return false;
        }

        return true;
    }

    /**
     * @return array Values to save: [Option name => New value].
     *
     * @since 1.0
     */
    protected function parseRequest()
    {
        $values = [];

        foreach ($this->fields as $optionName => $field) {
            if (!isset($_POST[$optionName])) {
                continue;
            }

            $field->setValue( sanitize_text_field( wp_unslash( $_POST[$optionName] ) ), 'validate');
            $newValue = $field->getValue('save');

            // Save all values, not just the new ones
            $values[$optionName] = $newValue;
        }

        return $values;
    }

    /**
     * @access protected
     *
     * @since 1.0
     */
    public function display()
    {
        echo '<div class="wrap">';
            echo $this->renderTabs(); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped

            if ($this->saved) {
                echo $this->renderNotice(); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
            }

            echo $this->renderForm(); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
        echo '</div>';
    }

    /**
     * @return string
     *
     * @since 1.0
     */
    protected function renderTabs()
    {
        $tabs = mpapp()->bundles()->settings()->getCategories();

        $currentTab = $this->getCurrentTab();
        $currentSection = $this->getCurrentSection();

        $output = '<h1 class="nav-tab-wrapper">';
            foreach ($tabs as $tab => $label) {
                $tabUrl = $this->getUrl(['tab' => $tab]);

                if ($tab != $currentTab) {
                    $output .= mpa_tmpl_link($tabUrl, $label, ['class' => 'nav-tab']);
                } else if (!empty($currentSection)) {
                    // Add link to the "parent" settings
                    $output .= mpa_tmpl_link($tabUrl, $label, ['class' => 'nav-tab nav-tab-active']);
                } else {
                    $output .= '<span class="nav-tab nav-tab-active">' . esc_html($label) . '</span>';
                }
            }
        $output .= '</h1>';

        return $output;
    }

    /**
     * @return string
     *
     * @since 1.0
     */
    protected function renderNotice()
    {
        return mpa_tmpl_notice('success', esc_html__('Settings saved.', 'motopress-appointment'));
    }

    /**
     * @return string
     *
     * @since 1.0
     */
    protected function renderForm()
    {
        $currentTab = $this->getCurrentTab();

        $output = '<form action="' . esc_url($this->getUrl()) . '" method="POST">';
            $output .= wp_nonce_field("mpa_save_{$currentTab}_settings", "_mpa_{$currentTab}_settings_nonce", true, false);

            $output .= mpa_render_template('private/fields/form-table.php', ['fields' => $this->fields]);

            $output .= '<div class="mpa-settings-tab-actions">';
                $output .= '<p class="submit">';
                    $output .= '<input type="submit" name="mpa_save_settings" class="button button-primary" value="' . esc_html__('Save Changes', 'motopress-appointment') . '">';
                $output .= '</p>';
            $output .= '</div>';
        $output .= '</form>';

        return $output;
    }

    /**
     * @param string $tab Optional. Current tab by default.
     * @param string $section Optional. No section by default.
     * @return array
     *
     * @since 1.1.0
     */
    public function getFields($tab = '', $section = '')
    {
        if (empty($tab)) {
            $tab = $this->getCurrentTab();
        }

        if (empty($section)) {
            return mpapp()->bundles()->settings()->getSettings($tab);
        } else {
            return apply_filters("mpa_{$tab}_section_settings", [], $section);
        }
    }

    /**
     * @return string
     *
     * @since 1.0
     */
    public function getCurrentTab()
    {
        if (is_null($this->currentTab)) {
            if (isset($_GET['tab'])) {
                $currentTab = wp_strip_all_tags( wp_unslash( $_GET['tab'] ) );
            } else {
                $currentTab = 'general';
            }

            $tabs = mpapp()->bundles()->settings()->getCategories();

            if (!array_key_exists($currentTab, $tabs)) {
                $currentTab = mpa_first_key($tabs);
            }

            $this->currentTab = $currentTab;
        }

        return $this->currentTab;
    }

    /**
     * @return string
     *
     * @since 1.1.0
     */
    public function getCurrentSection()
    {
        if (is_null($this->currentSection)) {
            if (isset($_GET['section'])) {
                $currentSection = wp_strip_all_tags( wp_unslash( $_GET['section'] ) );
            } else {
                $currentSection = '';
            }

            $this->currentSection = $currentSection;
        }

        return $this->currentSection;
    }

    /**
     * @return string
     *
     * @since 1.0
     */
    protected function getPageTitle()
    {
        return esc_html__('Settings', 'motopress-appointment');
    }

    /**
     * @return string
     *
     * @since 1.0
     */
    protected function getMenuTitle()
    {
        return esc_html__('Settings', 'motopress-appointment');
    }

    /**
     * @param array $additionalArgs Optional.
     * @return string
     *
     * @since 1.0
     */
    public function getUrl($additionalArgs = [])
    {
        $moreArgs = [];

        // Add tab and section to URL
        if ($this->isCurrentPage()) {
            $currentTab = $this->getCurrentTab(); // Not empty ('')

            if ($currentTab != 'general') {
                $moreArgs['tab'] = $currentTab;

                // Add section to URL
                $currentSection = $this->getCurrentSection();

                // But don't add section to the URL if making an URL of random tab
                if (!empty($currentSection) && !isset($additionalArgs['tab'])) {
                    $moreArgs['section'] = $currentSection;
                }
            }
        }

        return parent::getUrl(array_merge($moreArgs, $additionalArgs));
    }
}
