<?php

namespace MotoPress\Appointment\AdminPages\Manage;

if (!defined('ABSPATH')) {
    exit;
}

/**
 * @since 1.0
 */
class ManagePostsPage
{
    /**
     * @var string
     *
     * @since 1.0
     */
    protected $postType;

    /**
     * @param string $postType
     *
     * @since 1.0
     */
    public function __construct($postType)
    {
        $this->postType = $postType;

        add_action('admin_init', [$this, 'onLoad']);
    }

    /**
     * @access protected
     *
     * @since 1.0
     */
    public function onLoad()
    {
        if ($this->isCurrentPage()) {
            $this->addActions();
        }
    }

    /**
     * @since 1.0
     */
    protected function addActions()
    {
        add_filter("manage_{$this->postType}_posts_columns", [$this, 'filterColumns']);
        add_filter("manage_edit-{$this->postType}_sortable_columns", [$this, 'filterSortableColumns']);
        add_action("manage_{$this->postType}_posts_custom_column", [$this, 'manageCustomColumn'], 10, 2);
        add_action('admin_enqueue_scripts', [$this, 'maybeEnqueueScripts']);
    }

    /**
     * @param array $columns
     * @return array
     *
     * @access protected
     *
     * @since 1.0
     */
    public function filterColumns($columns)
    {
        $customColumns = $this->customColumns();

        // Add custom columns
        if (!empty($customColumns)) {
            $columns = array_merge($columns, $customColumns);

            // Make 'Date' columns last
            if (isset($columns['date'])) {
                $dateLabel = $columns['date'];
                unset($columns['date']);
                $columns['date'] = $dateLabel;
            }
        }

        return $columns;
    }

    /**
     * @param array $columns
     * @return array
     *
     * @access protected
     *
     * @since 1.0
     */
    public function filterSortableColumns($columns)
    {
        return array_merge($columns, $this->customSortableColumns());
    }

    /**
     * Fires only for custom columns.
     *
     * @param string $columnName
     * @param int $postId
     *
     * @access protected
     *
     * @since 1.0
     */
    public function manageCustomColumn($columnName, $postId)
    {
        $entity = mpapp()->repositories()->getByPostType($this->postType)->findById($postId);

        if (!is_null($entity)) {
            $this->displayValue($columnName, $entity);
        } else {
            echo esc_html( mpa_tmpl_placeholder() );
        }
    }

    /**
     * @param string $columnName Only custom columns.
     * @param \MotoPress\Appointment\Entities\AbstractEntity $entity
     * @return string
     *
     * @since 1.0
     */
    protected function displayValue($columnName, $entity)
    {
        echo esc_html( mpa_tmpl_placeholder() );
    }

    /**
     * @access protected
     *
     * @since 1.0
     */
    public function maybeEnqueueScripts()
    {
        if ($this->isCurrentPage()) {
            $this->enqueueScripts();
        }
    }

    /**
     * @since 1.0
     */
    protected function enqueueScripts() {}

    /**
     * @return array
     *
     * @since 1.0
     */
    protected function customColumns()
    {
        return [];
    }

    /**
     * @return array
     *
     * @since 1.0
     */
    protected function customSortableColumns()
    {
        return [];
    }

    /**
     * @return bool
     *
     * @global string $pagenow
     * @global string|null $typenow Is null if called too early.
     *
     * @since 1.0
     */
    public function isCurrentPage()
    {
        global $pagenow, $typenow;

        return is_admin() && $pagenow === 'edit.php' && $typenow === $this->postType;
    }

    /**
     * @return bool
     *
     * @since 1.0
     */
    public function isCurrentTrashPage()
    {
        return $this->isCurrentPage() && isset($_GET['post_status']) && $_GET['post_status'] === 'trash';
    }

    /**
     * @param array $additionalArgs Optional.
     * @return string
     *
     * @since 1.0
     */
    public function getUrl($additionalArgs = [])
    {
        $args = array_merge([
            'post_type' => $this->postType
        ], $additionalArgs);

        $url = add_query_arg($args, admin_url('edit.php'));

        return $url;
    }
}
