<?php

namespace MotoPress\Appointment\Bundles;

use MotoPress\Appointment\ListTables;
use MotoPress\Appointment\PostTypes\Statuses\BookingStatuses;

if (!defined('ABSPATH')) {
    exit;
}

/**
 * @since 1.0
 */
class SettingsBundle
{
    /**
     * @var array [Category name => Category label]
     *
     * @since 1.0
     */
    protected $categories = [];

    /**
     * @var array [Category => Fields]
     *
     * @since 1.0
     */
    protected $settings = [];

    /**
     * @return array [Category name => Category label]
     *
     * @since 1.0
     */
    public function getCategories()
    {
        if (empty($this->categories)) {
            $categories = [
                'general' => esc_html__('General', 'motopress-appointment'),
                'email'   => esc_html__('Emails', 'motopress-appointment')
            ];

            /** @since 1.0 */
            $categories = apply_filters('mpa_settings_categories', $categories);

            $this->categories = $categories;
        }

        return $this->categories;
    }

    /**
     * @param string $category
     * @return array Category fields.
     *
     * @since 1.0
     */
    public function getSettings($category)
    {
        if (!array_key_exists($category, $this->settings)) {
            $method = "{$category}Settings";

            if (method_exists($this, $method)) {
                $settings = $this->$method();
            } else {
                $settings = [];
            }

            /** @since 1.0 */
            $settings = apply_filters("mpa_{$category}_settings", $settings);

            $this->settings[$category] = $settings;
        }

        return $this->settings[$category];
    }

    /**
     * @return array
     *
     * @since 1.0
     */
    protected function generalSettings()
    {
        return [
			// General setting
            'default_time_step' => [
                'type'        => 'number',
                'label'       => esc_html__('Default Time Step', 'motopress-appointment'),
                'description' => esc_html__('Default length of the time slot step, which is used for the business hours and appointment durations.', 'motopress-appointment'),
                'min'         => 1,
                'default'     => 30,
                'size'        => 'small'
            ],
            'default_booking_status' => [
                'type'        => 'select',
                'label'       => esc_html__('Default Appointment Status', 'motopress-appointment'),
                'description' => esc_html__('Select status for newly booked appointments.', 'motopress-appointment'),
                'options'     => [
                    BookingStatuses::STATUS_PENDING   => esc_html_x('Pending', 'Booking status', 'motopress-appointment'),
                    BookingStatuses::STATUS_CONFIRMED => esc_html_x('Confirmed', 'Booking status', 'motopress-appointment')
                ],
                'default'     => BookingStatuses::STATUS_CONFIRMED,
                'size'        => 'regular'
            ],
			'allow_multibooking' => [
				'type'			=> 'checkbox',
				'label'			=> esc_html__('Allow Multibooking', 'motopress-appointment'),
				'label2'		=> esc_html__('Allow clients to book more than one service at a time.', 'motopress-appointment'),
				'default'		=> false,
			],

			// Misc group
			'misc_settings' => [
				'type'			=> 'group',
				'label'			=> esc_html__('Misc', 'motopress-appointment'),
			],
            'currency' => [
                'type'        => 'select',
                'label'       => esc_html__('Currency', 'motopress-appointment'),
                'options'     => mpapp()->bundles()->currencies()->getCurrencies(),
                'default'     => 'EUR',
                'size'        => 'regular'
            ],
            'currency_position' => [
                'type'        => 'select',
                'label'       => esc_html__('Currency Position', 'motopress-appointment'),
                'options'     => mpapp()->bundles()->currencies()->getPositions(),
                'default'     => 'before',
                'size'        => 'regular'
            ],
            'decimal_separator' => [
                'type'        => 'text',
                'label'       => esc_html__('Decimal Separator', 'motopress-appointment'),
                'default'     => '.',
                'size'        => 'small'
            ],
            'thousand_separator' => [
                'type'        => 'text',
                'label'       => esc_html__('Thousand Separator', 'motopress-appointment'),
                'default'     => ',',
                'size'        => 'small'
            ],
            'number_of_decimals' => [
                'type'        => 'number',
                'label'       => esc_html__('Number of Decimals', 'motopress-appointment'),
                'default'     => 2,
                'size'        => 'small'
            ]
        ];
    }

    /**
     * @return array
     *
     * @since 1.1.0
     */
    protected function emailSettings()
    {
        return [
            // Admin emails group
            'admin_emails_group' => [
                'type'         => 'group',
                'label'        => esc_html__('Admin Emails', 'motopress-appointment')
            ],
            'admin_emails' => [
                'type'         => 'list-table',
                'list_table'   => new ListTables\Emails\AdminEmailsListTable('admin_emails')
            ],

            // Customer emails group
            'customer_emails_group' => [
                'type'         => 'group',
                'label'        => esc_html__('Customer Emails', 'motopress-appointment')
            ],
            'customer_emails' => [
                'type'         => 'list-table',
                'list_table'   => new ListTables\Emails\CustomerEmailsListTable('customer_emails')
            ],

            // Email sender group
            'email_sender_group' => [
                'type'         => 'group',
                'label'        => esc_html__('Email Sender', 'motopress-appointment')
            ],
            'admin_email' => [
                'type'         => 'email',
                'label'        => esc_html__('Administrator Email', 'motopress-appointment'),
                'placeholder'  => mpapp()->settings()->getDefaultAdminEmail(),
                'size'         => 'regular'
            ],
            'from_email' => [
                'type'         => 'email',
                'label'        => esc_html__('From Email', 'motopress-appointment'),
                'placeholder'  => mpapp()->settings()->getAdminEmail(),
                'size'         => 'regular',
                'translatable' => true
            ],
            'from_name' => [
                'type'         => 'text',
                'label'        => esc_html__('From Name', 'motopress-appointment'),
                'placeholder'  => mpapp()->settings()->getDefaultFromName(),
                'size'         => 'regular',
                'translatable' => true
            ],

            // Email template group
            'email_template_group' => [
                'type'         => 'group',
                'label'        => esc_html__('Email Template', 'motopress-appointment')
            ],
            'email_logo_url' => [
                'type'         => 'text',
                'label'        => esc_html__('Logo URL', 'motopress-appointment'),
                'description'  => esc_html__('URL to an image you want to show in the email header.', 'motopress-appointment'),
                'placeholder'  => mpapp()->settings()->getDefaultEmailLogoUrl(),
                'size'         => 'large',
                'translatable' => true
            ],
            'email_footer_text' => [
                'type'         => 'textarea',
                'label'        => esc_html__('Footer Text', 'motopress-appointment'),
                'description'  => esc_html__('Text applied to the footer of all emails.', 'motopress-appointment'),
                'rows'         => 3,
                'size'         => 'regular',
                'default'      => mpapp()->settings()->getDefaultEmailFooterText(),
                'translatable' => true
            ],
            'email_template_parts' => [
                'type'         => 'list-table',
                'label'        => esc_html__('Template Parts', 'motopress-appointment'),
                'list_table'   => new ListTables\Emails\TemplatePartsListTable('email_template_parts')
            ],

            // Email styles group
            'email_styles_group' => [
                'type'         => 'group',
                'label'        => esc_html__('Styles', 'motopress-appointment')
            ],
            'email_base_color' => [
                'type'         => 'color-picker',
                'label'        => esc_html__('Base Color', 'motopress-appointment'),
                'default'      => '#557da1'
            ],
            'email_bg_color' => [
                'type'         => 'color-picker',
                'label'        => esc_html__('Background Color', 'motopress-appointment'),
                'default'      => '#f5f5f5'
            ],
            'email_body_bg_color' => [
                'type'         => 'color-picker',
                'label'        => esc_html__('Body Background Color', 'motopress-appointment'),
                'default'      => '#fdfdfd'
            ],
            'email_text_color' => [
                'type'         => 'color-picker',
                'label'        => esc_html__('Body Text Color', 'motopress-appointment'),
                'default'      => '#505050'
            ]
        ];
    }

}
