<?php

namespace MotoPress\Appointment\Emails;

use MotoPress\Appointment\Utils\EmailUtils;

if (!defined('ABSPATH')) {
    exit;
}

/**
 * @since 1.1.0
 */
class EmailTags
{
    /**
     * @var Tags\AbstractTagsGroup[] [Group name => AbstractTagsGroup]
     *
     * @since 1.1.0
     */
    protected $tagGroups = [];

    /**
     * @param Tags\AbstractTagsGroup[] $tagGroups
     *
     * @since 1.1.0
     */
    public function __construct($tagGroups)
    {
        foreach ($tagGroups as $tagsGroup) {
            $this->addGroup($tagsGroup);
        }
    }

    /**
     * @param Tags\AbstractTagsGroup $tagsGroup
     *
     * @since 1.1.0
     */
    public function addGroup($tagsGroup)
    {
        $this->tagGroups[$tagsGroup->getName()] = $tagsGroup;
    }

    /**
     * @param string $name
     * @return Tags\AbstractTagsGroup|null
     *
     * @since 1.1.0
     */
    public function getGroup($name)
    {
        if ($this->hasGroup($name)) {
            return $this->tagGroups[$name];
        } else {
            return null;
        }
    }

    /**
     * @return Tags\AbstractTagsGroup[] [Group name => AbstractTagsGroup]
     *
     * @since 1.1.0
     */
    public function getGroups()
    {
        return $this->tagGroups;
    }

    /**
     * @param string $name
     * @return bool
     *
     * @since 1.1.0
     */
    public function hasGroup($name)
    {
        return array_key_exists($name, $this->tagGroups);
    }

    /**
     * @param string $name
     *
     * @since 1.1.0
     */
    public function removeGroup($name)
    {
        if ($this->hasGroup($name)) {
            unset($this->tagGroups[$name]);
        }
    }

    /**
	 * @param array $args Optional.
	 *		@param bool $args['nested']			Add "nested" tags (see AbstractTagsGroup
	 *											for details). False by default.
	 *		@param bool $args['single_booking']	Add tags that intended for use only
	 *											with single booking. True by default.
     * @return string
     *
     * @since 1.1.0
	 * @since 1.4.0 the argument <code>$getNested</code> was replaced by <code>$args</code>.
     */
    public function getTagsDescription($args = [])
    {
		$args += [
			'nested'			=> false,
			'single_booking'	=> true,
		];

        $description = '';

        foreach ($this->tagGroups as $tagsGroup) {
			if (!EmailUtils::isSuitableTagsGroupToDisplay($tagsGroup, $args)) {
				continue;
			}

            // Add label
            $description .= '<b>' . $tagsGroup->getLabel() . '</b>:<br>';

            // Add tags
			$actualTags = '';
			$deprecatedTags = '';

            foreach ($tagsGroup->getTags() as $tagName => $tagArgs) {
                $tag = mpa_tmpl_tag($tagName, $tagArgs);
				$tag .= '<br>'; // Add tags separator

				$tagArgs['deprecated'] ? $deprecatedTags .= $tag : $actualTags .= $tag;
            }

			$description .= $actualTags . $deprecatedTags;

            // Add groups separator
            $description .= '<br>';
        }

        // Remove last '...<br><br>' (groups separator + tags separator)
        if (!empty($description)) {
            $description = substr($description, 0, -8);
        }

        return $description;
    }

    /**
     * @param string $content
     * @param array $args Optional.
     *     @param \MotoPress\Appointment\Entities\Booking|null $args['booking']
     *     @param \MotoPress\Appointment\Entities\Payment|null $args['payment']
     * @return string
     *
     * @since 1.1.0
     */
    public function replaceTags($content, $args = [])
    {
        // Add default args
        $args += ['booking' => null, 'payment' => null, 'reservation' => null];

        // Select the first available reservation, so employee, location, service
        // and reservation tags can be used not only within {reservation_details}
        // template, but also in the general email templates
        if (!is_null($args['booking']) && empty($args['reservation'])) {
            $args['reservation'] = reset($args['booking']->reservations);
        }

        // Replace tags
        foreach ($this->tagGroups as $tagsGroup) {
            $content = $tagsGroup->replaceTags($content, $args, $this);
        }

        return $content;
    }
}
