<?php

namespace MotoPress\Appointment\Emails;

use MotoPress\Appointment\Entities;
use MotoPress\Appointment\PostTypes\Statuses\BookingStatuses;

if (!defined('ABSPATH')) {
    exit;
}

/**
 * @since 1.1.0
 */
class EmailsDispatcher
{
    /**
     * By default loads on "plugins_loaded" (15).
     *
     * @see \MotoPress\Appointment\Plugin::load()
     *
     * @since 1.1.0
     */
    public function load()
    {
        $this->addActions();
    }

    /**
     * @since 1.1.0
     */
    protected function addActions()
    {
        // Filter settings
        add_filter('mpa_email_section_settings', [$this, 'filterSettings'], 10, 2);

        // Listen booking actions
        add_action('mpa_booking_placed_by_user', [$this, 'triggerNewBooking'], 10, 1);
        add_action('mpa_booking_status_changed', [$this, 'triggerBookingUpdate'], 10, 3);
    }

    /**
     * @param array $fields
     * @param string $sectionName
     * @return array
     *
     * @access protected
     *
     * @since 1.1.0
     */
    public function filterSettings($fields, $sectionName)
    {
        $email = mpapp()->emails()->getEmailByName($sectionName);
        $templatePart = mpapp()->templates()->getEmailTemplatePartByName($sectionName);

        if (!is_null($email)) {
            $fields += $email->getSettingsFields();
        } else if (!is_null($templatePart)) {
            $fields += $templatePart->getFields();
        }

        return $fields;
    }

    /**
     * @param Entities\Booking $booking
     *
     * @access protected
     *
     * @see BookingStatuses::notifyTransition()
     *
     * @since 1.1.0
     */
    public function triggerNewBooking($booking)
    {
        switch (mpapp()->settings()->getDefaultBookingStatus()) {
            case BookingStatuses::STATUS_CONFIRMED:
                $this->triggerEmail(mpapp()->emails()->adminNewBooking(), $booking);
                $this->triggerEmail(mpapp()->emails()->customerNewBooking(), $booking);
                break;

            case BookingStatuses::STATUS_PENDING:
                $this->triggerEmail(mpapp()->emails()->adminPendingBooking(), $booking);
                $this->triggerEmail(mpapp()->emails()->customerPendingBooking(), $booking);
                break;
        }
    }

    /**
     * @param Entities\Booking $booking
     * @param string $newStatus
     * @param string $oldStatus
     *
     * @access protected
     *
     * @see BookingStatuses::notifyTransition()
     *
     * @since 1.1.0
     */
    public function triggerBookingUpdate($booking, $newStatus, $oldStatus)
    {
        switch ($newStatus) {
            case BookingStatuses::STATUS_CONFIRMED:
                $this->triggerEmail(mpapp()->emails()->customerApprovedBooking(), $booking);
                break;

            case BookingStatuses::STATUS_CANCELLED:
                $this->triggerEmail(mpapp()->emails()->customerCancelledBooking(), $booking);
                break;
        }
    }

    /**
     * @param AbstractEmail $email
     * @param Entities\Booking $booking
     * @param array $args Optional.
     *     @param Entities\Payment $args['payment'] Null by default.
     *     @param bool $args['is_test'] False by default.
     * @return bool
     *
     * @since 1.1.0
     */
    public function triggerEmail($email, $booking, $args = [])
    {
        // Fill optional args
        $args += [
            'payment' => null,
            'is_test' => false
        ];

        $payment = $args['payment'];
        $isTest  = $args['is_test'];

        // Check if the email is disabled
        if ($email->isDisabled() && !$isTest) {
            return false;
        }

        // Init email with booking and payment
        $email->setupEntities($booking, $payment);

        // Do we have any receiver?
        if (!$this->checkRecipients($email, $isTest)) {
            if (!$isTest) {
                $this->log($booking, $this->recipientsError($email));
            }

            return false;
        }

        // Send email
        $isSended = $this->sendEmail($email, $booking, $args);

        if (!$isTest) {
            $this->log($booking, $this->sendedMessage($email, $isSended));
        }

        return $isSended;
    }

    /**
     * @param AbstractEmail $email
     * @param bool $isTest Optional. False by default.
     * @return bool
     *
     * @since 1.1.0
     */
    protected function checkRecipients($email, $isTest = false)
    {
        $recipients = $this->getRecipients($email, $isTest);

        return !empty($recipients);
    }

    /**
     * @param AbstractEmail $email
     * @param bool $isTest Optional. False by default.
     * @return string
     *
     * @since 1.1.0
     */
    protected function getRecipients($email, $isTest = false)
    {
        if ($isTest) {
            return mpapp()->settings()->getAdminEmail();
        } else {
            return $email->getRecipients();
        }
    }

    /**
     * @param AbstractEmail $email
     * @param Entities\Booking $booking
     * @param array $args Optional.
     *     @param Entities\Payment $args['payment'] Null by default.
     *     @param bool $args['is_test'] False by default.
     * @return bool
     *
     * @since 1.1.0
     */
    public function sendEmail($email, $booking, $args = [])
    {
        // Fill optional args
        $args += [
            'payment' => null,
            'is_test' => false
        ];

        // Change the language
        $emailType = $this->typeOfEmail($email);

        if (!$args['is_test']) { // Use default language for tests
            /**
             * mpa_set_language_before_send_admin_email
             * mpa_set_language_before_send_customer_email
             * mpa_set_language_before_send_other_email
             *
             * @since 1.1.0
             */
            do_action("mpa_set_language_before_send_{$emailType}_email", $email, $booking, $args);
        }

        // Send email
        $mailer = mpa_mailer();
        $sendTo = $this->getRecipients($email, $args['is_test']);

        /** @since 1.1.0 */
        do_action('mpa_before_send_email', $email, $booking, $args);

        $isSended = $mailer->send($sendTo, $email->getSubject(), $email->render());

        /** @since 1.1.0 */
        do_action('mpa_after_send_email', $email, $booking, $args);

        return $isSended;
    }

    /**
     * @param AbstractEmail $email
     * @return string admin|customer|other
     *
     * @since 1.1.0
     */
    protected function typeOfEmail($email)
    {
        if ($email instanceof Admin\AbstractAdminEmail) {
            return 'admin';
        } else if ($email instanceof Customer\AbstractCustomerEmail) {
            return 'customer';
        } else {
            return 'other';
        }
    }

    /**
     * @param Entities\Booking $booking
     * @param string $message
     *
     * @since 1.1.0
     */
    protected function log($booking, $message)
    {
        if (empty($message)) {
            return;
        }

        mpa_add_booking_log($booking, $message, $this->getLogAuthor());
    }

    /**
     * @return int|null
     *
     * @since 1.1.0
     */
    protected function getLogAuthor()
    {
        // Null means "define automatically". Some addons may change it
        return null;
    }

    /**
     * @param AbstractEmail $email
     * @return string
     *
     * @since 1.1.0
     */
    protected function recipientsError($email)
    {
        // Translators: %s: Email name, like "New Booking Email".
        $error = esc_html__('"%s" email will not be sent: there are no recipients.', 'motopress-appointment');

        return sprintf($error, $email->getLabel());
    }

    /**
     * @param AbstractEmail $email
     * @param bool $isSended
     * @return string
     *
     * @since 1.1.0
     */
    protected function sendedMessage($email, $isSended)
    {
        $message = '';

        $emailType = $this->typeOfEmail($email);

        if ($isSended) {
            if ($emailType == 'admin') {
                // Translators: %s: Email name, like "New Booking Email".
                $message = sprintf(esc_html__('"%s" email was sent to admin.', 'motopress-appointment'), $email->getLabel());
            } else if ($emailType == 'customer') {
                // Translators: %s: Email name, like "New Booking Email".
                $message = sprintf(esc_html__('"%s" email was sent to customer.', 'motopress-appointment'), $email->getLabel());
            } else {
                // Translators: %s: Email name, like "New Booking Email".
                $message = sprintf(esc_html__('"%s" email was sent.', 'motopress-appointment'), $email->getLabel());
            }
        } else {
            if ($emailType == 'admin') {
                // Translators: %s: Email name, like "New Booking Email".
                $message = sprintf(esc_html__('"%s" mail sending to admin is failed.', 'motopress-appointment'), $email->getLabel());
            } else if ($emailType == 'customer') {
                // Translators: %s: Email name, like "New Booking Email".
                $message = sprintf(esc_html__('"%s" mail sending to customer is failed.', 'motopress-appointment'), $email->getLabel());
            } else {
                // Translators: %s: Email name, like "New Booking Email".
                $message = sprintf(esc_html__('"%s" mail sending is failed.', 'motopress-appointment'), $email->getLabel());
            }
        }

        return $message;
    }
}
