<?php

namespace MotoPress\Appointment\Emails\Tags;

if (!defined('ABSPATH')) {
    exit;
}

/**
 * @since 1.1.0
 */
abstract class AbstractTagsGroup
{
    /**
     * @var bool See constructor.
     *
     * @since 1.1.0
     */
    protected $isNested = false;

    /**
     * @var array [Tag name => [description]]
     *
     * @since 1.1.0
     */
    protected $tags = [];

    /**
     * @param bool $isNested Optional. Is intended for nested templates, like
     *     admin reservation details template part etc. False by default.
     *
     * @since 1.1.0
     */
    public function __construct($isNested = false)
    {
        $this->isNested = $isNested;

        $this->setupTags();
    }

    /**
     * @return string
     *
     * @since 1.1.0
     */
    abstract public function getName();

    /**
     * @return string
     *
     * @since 1.1.0
     */
    abstract public function getLabel();

    /**
     * @since 1.1.0
     */
    abstract protected function setupTags();

    /**
     * @return bool
     *
     * @since 1.1.0
     */
    public function isNested()
    {
        return $this->isNested;
    }

    /**
     * @param string $tag
     * @return bool
     *
     * @since 1.1.0
     */
    public function hasTag($tag)
    {
        return array_key_exists($tag, $this->tags);
    }

    /**
     * @param string $tag
     * @param mixed $default
     * @return mixed
     *
     * @since 1.1.0
     */
    public function getTag($tag, $default = [])
    {
        if ($this->hasTag($tag)) {
            return $this->tags[$tag];
        } else {
            return $default;
        }
    }

    /**
     * @return array
     *
     * @since 1.1.0
     */
    public function getTags()
    {
        return $this->tags;
    }

    /**
     * @param array $tags
     *
     * @since 1.1.0
     */
    public function addTags($tags)
    {
        foreach ($tags as $name => $args) {
            $this->tags[$name] = array_merge([
                'description' => '',
                'deprecated'  => false,
            ], $args);
        }
    }

    /**
     * @param string[] $tags
     *
     * @since 1.1.0
     */
    public function removeTags($tags)
    {
        foreach ($tags as $tag) {
            if ($this->hasTag($tag)) {
                unset($this->tags[$tag]);
            }
        }
    }

    /**
     * @param string $content
     * @param array $args
     *     @param \MotoPress\Appointment\Entities\Booking|null $args['booking']
     *     @param \MotoPress\Appointment\Entities\Payment|null $args['payment']
     * @param \MotoPress\Appointment\Emails\EmailTags $tagsControl
     * @return string
     *
     * @since 1.1.0
     */
    public function replaceTags($content, $args, $tagsControl)
    {
        if (empty($this->tags)) {
            return $content;
        }

        $content = preg_replace_callback($this->getTagsRegex(), function ($match) use ($args, $tagsControl, $content) {
            $tag = str_replace(['{', '}'], '', $match[0]);
            $tag = strtolower($tag); // 'Appointment' -> 'appointment'

            return $this->replaceTag($tag, $args, $tagsControl, $content);
        }, $content);

        return $content;
    }

    /**
     * @param string $tag Tag name (lowercased).
     * @param array $args
     *     @param \MotoPress\Appointment\Entities\Booking|null $args['booking']
     *     @param \MotoPress\Appointment\Entities\Payment|null $args['payment']
     * @param \MotoPress\Appointment\Emails\EmailTags $tagsControl
     * @param string $content
     *
     * @since 1.1.0
     */
    abstract protected function replaceTag($tag, $args, $tagsControl, $content);

    /**
     * @return string Regex to match all tags from current group.
     *
     * @since 1.1.0
     */
    public function getTagsRegex()
    {
        $tags = array_keys($this->tags);

        // Use case insensitive regex. See {Appointment} in email
        // footer text, for example
        $regex = '/\{' . implode('\}|\{', $tags) . '\}/i';

        return $regex;
    }
}
