<?php

namespace MotoPress\Appointment\Fields;

if (!defined('ABSPATH')) {
    exit;
}

/**
 * @since 1.0
 */
abstract class AbstractField
{
    /** @since 1.0 */
    const TYPE = 'abstract';

    /**
     * @var string Without prefix.
     *
     * @since 1.0
     */
    protected $name = '';

    /**
     * @var string With prefix.
     *
     * @since 1.0
     */
    protected $inputName = '';

    /**
     * @var string Like the input name, but all '_' replaced with '-'.
     *
     * @since 1.0
     */
    protected $inputId = '';

    /**
     * @var string
     *
     * @since 1.0
     */
    public $label = '';

    /**
     * @var string
     *
     * @since 1.0
     */
    public $description = '';

    /**
     * @var mixed
     *
     * @since 1.0
     */
    protected $value = '';

    /**
     * @var mixed
     *
     * @since 1.0
     */
    protected $default = '';

    /**
     * @var string Additional control classes.
     *
     * @since 1.0
     */
    protected $class = '';

    /**
     * @var string
     *
     * @since 1.0
     */
    protected $sizeClass = '';

    /**
     * @var array
     *
     * @since 1.0
     */
    protected $atts = [];

    /**
     * @var bool
     *
     * @since 1.3
     */
    public $isInline = false;

    /**
     * @param string $inputName Prefixed name.
     * @param array $args
     * @param mixed $value Optional. Null by default.
     *
     * @since 1.0
     */
    public function __construct($inputName, $args, $value = null)
    {
        $this->setName($inputName);
        $this->setupArgs($args);

        if (!is_null($value)) {
            $this->setValue($value);
        }
    }

    /**
     * @param array $args
     *
     * @since 1.0
     */
    protected function setupArgs($args)
    {
        // Set most values
        foreach ($this->mapFields() as $parameter => $field) {
            if (array_key_exists($parameter, $args)) {
                $this->$field = $args[$parameter];
            }
        }

        // Set $default
        if (isset($args['default'])) {
            $this->setDefault($args['default']);
        }

        // Set up the initial value
        $this->value = $this->default;

        // Set size
        if (isset($args['size'])) {
            $this->setSize($args['size']);
        }
    }

    /**
     * @return array
     *
     * @since 1.0
     */
    protected function mapFields()
    {
        return [
            // Parameter name => Field name
            // inner_label    => innerLabel
            'label'       => 'label',
            'description' => 'description',
            'class'       => 'class',
            'atts'        => 'atts',
            'inline'      => 'isInline'
        ];
    }

    /**
     * @param mixed $value
     * @return mixed
     *
     * @since 1.0
     */
    protected function validateValue($value)
    {
        return sanitize_text_field($value);
    }

    /**
     * @param string $inputName Prefixed name.
     *
     * @since 1.0
     */
    public function setName($inputName)
    {
        $this->name      = mpa_unprefix($inputName);
        $this->inputName = $inputName;
        $this->inputId   = mpa_tmpl_id($inputName, true); // Underscores are important for widget
                                                          // field names: "my-widget[__i__][title]"
    }

    /**
     * @param string $size small|regular|mild|large|half|wide
     *
     * @since 1.0
     */
    public function setSize($size)
    {
        switch ($size) {
            case 'small':   $this->sizeClass = 'small-text';     break;
            case 'mild':    $this->sizeClass = 'all-options';    break;
            case 'regular': $this->sizeClass = 'regular-text';   break;
            case 'large':   $this->sizeClass = 'large-text';     break;
            case 'half':    $this->sizeClass = 'mpa-half-width'; break;
            case 'wide':    $this->sizeClass = 'widefat';        break;
            case 'none':
            default:        $this->sizeClass = '';               break;
        }
    }

    /**
     * @param mixed $value
     * @param mixed $validate Optional. False by default.
     *
     * @since 1.0
     */
    public function setValue($value, $validate = false)
    {
        if ($validate !== false) {
            $this->value = $this->validateValue($value);
        } else {
            // Metabox will always pass a single value if the array length = 1
            $this->value = $this->isSingle() ? $value : (array)$value;
        }
    }

    /**
     * @param mixed $default
     *
     * @since 1.0
     */
    public function setDefault($default)
    {
        $this->default = $default;
    }

    /**
     * @param string $context Optional. 'internal' by default. Variants:
     *     'internal' - for internal use (in the functions of the plugin);
     *     'save'     - prepare the value for the database.
     * @return mixed
     *
     * @since 1.0
     */
    public function getValue($context = 'internal')
    {
        return $this->value;
    }

    /**
     * @return string
     *
     * @since 1.0
     */
    public function hasLabel()
    {
        return $this->label !== '';
    }

    /**
     * @return string
     *
     * @since 1.1.0
     */
    public function hasDescription()
    {
        return $this->description !== '';
    }

    /**
     * @return string
     *
     * @since 1.0
     */
    public function renderLabel()
    {
        // Tip: use '&nbsp;' to output an empty label
        if ($this->hasLabel()) {
            return '<label for="' . esc_attr($this->inputId) . '">' . esc_html($this->label) . '</label>';
        } else {
            return '';
        }
    }

    /**
     * @return string
     *
     * @since 1.0
     */
    public function renderBody()
    {
        $wrapperTag = $this->getWrapperTag();

        ob_start();

        /** @since 1.0 */
        do_action('mpa_render_field_before_start', static::TYPE, $this);

        echo '<' . esc_html( $wrapperTag ) .
			mpa_tmpl_atts($this->controlAtts()) . '>'; // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped

            echo $this->renderInput(); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped

            /** @since 1.0 */
            do_action('mpa_render_field_after_input', static::TYPE, $this);

            echo wp_kses_post( $this->renderDescription() );

            /** @since 1.0 */
            do_action('mpa_render_field_before_end', static::TYPE, $this);
        echo '</' . esc_html( $wrapperTag ) . '>';

        /** @since 1.0 */
        do_action('mpa_render_field_after_end', static::TYPE, $this);

        $output = ob_get_clean();
        return $output;
    }

    /**
     * @param string $blockTag Optional. 'div' by default.
     * @param string $inlineTag Optional. 'span' by default.
     * @return string
     *
     * @since 1.3
     */
    protected function getWrapperTag($blockTag = 'div', $inlineTag = 'span')
    {
        return $this->isInline ? $inlineTag : $blockTag;
    }

    /**
     * @return string
     *
     * @since 1.0
     */
    abstract public function renderInput();

    /**
     * @return string
     *
     * @since 1.1.0
     */
    public function renderDescription()
    {
        if ($this->hasDescription()) {
            $wrapperTag = $this->getWrapperTag('p');

            return "<{$wrapperTag} class=\"description\">{$this->description}</{$wrapperTag}>";
        } else {
            return '';
        }
    }

    /**
     * @return array
     *
     * @since 1.0
     */
    protected function controlAtts()
    {
        return array_merge(
            [
                'class'          => rtrim('mpa-ctrl mpa-' . static::TYPE . '-ctrl ' . $this->class),
                'data-type'      => static::TYPE,
                'data-name'      => $this->name,
                'data-base-name' => $this->inputName
            ],
            $this->atts
        );
    }

    /**
     * @return array
     *
     * @since 1.0
     */
    protected function inputAtts()
    {
        return [
            'name'  => $this->inputName,
            'id'    => $this->inputId,
            'class' => $this->sizeClass
        ];
    }

    /**
     * @return bool Uses single or multiple postmetas.
     *
     * @since 1.0
     */
    public function isSingle()
    {
        return true;
    }

    /**
     * @return string
     *
     * @since 1.2
     */
    public function getType()
    {
        return static::TYPE;
    }
}
