<?php

use MotoPress\Appointment\Entities;

if (!defined('ABSPATH')) {
    exit;
}

/**
 * @param int|Booking $booking
 * @param string $message
 * @param int $author Optional. Null by default.
 * @return int|false The new comment's ID on success, false on failure.
 *
 * @since 1.0
 */
function mpa_add_booking_log($booking, $message, $author = null)
{
    return mpapp()->repositories()->booking()->addLog($booking, $message, $author);
}

/**
 * @param int $id Optional. Current post by default.
 * @param bool $forceReload Optional. False by default.
 * @return Entities\Booking|null
 *
 * @since 1.0
 * @since 1.2 $id is optional.
 */
function mpa_get_booking($id = 0, $forceReload = false)
{
    return mpapp()->repositories()->booking()->findById($id, $forceReload);
}

/**
 * @param array $args Optional.
 * @return array
 *
 * @see \MotoPress\Appointment\Repositories\AbstractRepository::findAll()
 *
 * @since 1.0
 */
function mpa_get_bookings($args = [])
{
    return mpapp()->repositories()->booking()->findAll($args);
}

/**
 * @param int $bookingId
 * @param bool $forceReload Optional. False by default.
 * @return Entities\Reservation[]
 *
 * @since 1.0
 */
function mpa_get_reservations($bookingId, $forceReload = false)
{
    $booking = mpa_get_booking($bookingId, $forceReload);
    return !is_null($booking) ? $booking->reservations : [];
}

/**
 * @param int $id Optional. Current post by default.
 * @param bool $forceReload Optional. False by default.
 * @return Entities\Employee|null
 *
 * @since 1.0
 * @since 1.2 $id is optional.
 */
function mpa_get_employee($id = 0, $forceReload = false)
{
    return mpapp()->repositories()->employee()->findById($id, $forceReload);
}

/**
 * @param array $args Optional.
 * @return array
 *
 * @see \MotoPress\Appointment\Repositories\AbstractRepository::findAll()
 *
 * @since 1.0
 */
function mpa_get_employees($args = ['fields' => ['id' => 'name']])
{
    return mpapp()->repositories()->employee()->findAll($args);
}

/**
 * @param int|int[] $location
 * @param array $args Optional.
 * @return array
 *
 * @since 1.2
 */
function mpa_get_employees_by_location($location, $args = ['fields' => ['id' => 'name']])
{
    $keyLocation = mpa_prefix('main_location', 'private');
    $keyEmployee = mpa_prefix('employee', 'private');

    $schedules = mpapp()->repositories()->schedule()->findAllByMeta($keyLocation, (array)$location, 'IN', ['fields' => 'ids']);
    $employees = [];

    // Get employees
    foreach ($schedules as $scheduleId) {
        $employeeId = get_post_meta($scheduleId, $keyEmployee, true);

        if (!empty($employeeId)) {
            $employees[] = intval($employeeId);
        }
    }

    // Get all required fields
    if ( !(count($args) == 1 && isset($args['fields']) && $args['fields'] === 'ids') ) {
        $args['post__in'] = $employees;

        $employees = mpapp()->repositories()->employee()->findAll($args);
    }

    return $employees;
}

/**
 * @param int|Entities\Employee $employee
 * @param array $args Optional.
 *     @param bool $args['show_contacts'] True by default.
 *     @param bool $args['show_social_networks'] True by default.
 *     @param bool $args['show_additional_info'] True by default.
 * @return array
 *
 * @since 1.2
 */
function mpa_get_employee_attributes($employee, $args = [])
{
    if (!is_object($employee) && !is_null($employee)) {
        $employee = mpa_get_employee($employee);
    }

    if (is_null($employee)) {
        return [];
    }

    $args += [
        'show_contacts'        => true,
        'show_social_networks' => true,
        'show_additional_info' => true
    ];

    $attributes = [];

    if ($args['show_contacts']) {
        $attributes = array_merge($attributes, $employee->contacts);
    }

    if ($args['show_social_networks']) {
        $attributes = array_merge($attributes, $employee->socialNetworks);
    }

    if ($args['show_additional_info']) {
        $attributes = array_merge($attributes, $employee->additionalInfo);
    }

    /**
     * @param array
     * @param Entities\Employee
     * @param array
     *
     * @since 1.2
     */
    $attributes = apply_filters('mpa_get_employee_attributes', $attributes, $employee, $args);

    return $attributes;
}

/**
 * @param int $id Optional. Current post by default.
 * @param bool $forceReload Optional. False by default.
 * @return Entities\Location|null
 *
 * @since 1.0
 * @since 1.2 $id is optional.
 */
function mpa_get_location($id = 0, $forceReload = false)
{
    return mpapp()->repositories()->location()->findById($id, $forceReload);
}

/**
 * @param array $args Optional.
 * @return array
 *
 * @see \MotoPress\Appointment\Repositories\AbstractRepository::findAll()
 *
 * @since 1.0
 */
function mpa_get_locations($args = ['fields' => ['id' => 'name']])
{
    return mpapp()->repositories()->location()->findAll($args);
}

/**
 * @param int $locationId Optional. 0 by default.
 * @param string|array $fields Optional. 'all', field name or key-value pair.
 *     ['slug' => 'name'] by default.
 * @param array $args Optional.
 * @return WP_Term[]|array
 *
 * @since 1.1.0
 */
function mpa_get_location_categories($locationId = 0, $fields = ['slug' => 'name'], $args = [])
{
    $taxonomy = MotoPress\Appointment\PostTypes\LocationPostType::CATEGORY_NAME;

    return mpa_get_terms($locationId, $taxonomy, $fields, $args);
}

/**
 * @param int $id Optional. Current post by default.
 * @param bool $forceReload Optional. False by default.
 * @return Entities\Schedule|null
 *
 * @since 1.0
 * @since 1.2 $id is optional.
 */
function mpa_get_schedule($id = 0, $forceReload = false)
{
    return mpapp()->repositories()->schedule()->findById($id, $forceReload);
}

/**
 * @param array $args Optional.
 * @return array
 *
 * @see \MotoPress\Appointment\Repositories\AbstractRepository::findAll()
 *
 * @since 1.0
 */
function mpa_get_schedules($args = ['fields' => ['id' => 'title']])
{
    return mpapp()->repositories()->schedule()->findAll($args);
}

/**
 * @param int $id Optional. Current post by default.
 * @param bool $forceReload Optional. False by default.
 * @return Entities\Service|null
 *
 * @since 1.0
 * @since 1.2 $id is optional.
 */
function mpa_get_service($id = 0, $forceReload = false)
{
    return mpapp()->repositories()->service()->findById($id, $forceReload);
}

/**
 * @param array $args Optional.
 * @return array
 *
 * @see \MotoPress\Appointment\Repositories\AbstractRepository::findAll()
 *
 * @since 1.0
 */
function mpa_get_services($args = ['fields' => ['id' => 'title']])
{
    return mpapp()->repositories()->service()->findAll($args);
}

/**
 * @param int|Entities\Service $service
 * @param array $args Optional.
 *     @param bool $args['show_price'] True by default.
 *     @param bool $args['show_duration'] True by default.
 * @return array Array for template post/attributes.php.
 *
 * @since 1.2
 */
function mpa_get_service_attributes($service, $args = [])
{
    if (!is_object($service) && !is_null($service)) {
        $service = mpa_get_service($service);
    }

    if (is_null($service)) {
        return [];
    }

    $args += [
        'show_price'    => true,
		'show_duration' => true,
		'show_capacity' => true,
    ];

    $attributes = [];

    if ($args['show_price']) {
        $attributes[] = [
            'label'   => esc_html__('Price', 'motopress-appointment'),
            'content' => mpa_tmpl_price($service->price)
        ];
    }

    if ($args['show_duration']) {
        $attributes[] = [
            'label'   => esc_html__('Duration', 'motopress-appointment'),
            'content' => mpa_minutes_to_duration($service->duration)
        ];
    }

	if ($args['show_capacity']) {
		$attributes[] = [
			'label'   => esc_html__('Capacity', 'motopress-appointment'),
			'content' => mpa_tmpl_service_capacity($service),
		];
	}

    /**
     * @param array
     * @param Entities\Service
     *
     * @since 1.2
     */
    $attributes = apply_filters('mpa_get_service_attributes', $attributes, $service);

    return $attributes;
}

/**
 * @param int $serviceId Optional. 0 by default.
 * @param string|array $fields Optional. 'all', field name or key-value pair.
 *     ['slug' => 'name'] by default.
 * @param array $args Optional.
 * @return WP_Term[]|array
 *
 * @since 1.0
 */
function mpa_get_service_categories($serviceId = 0, $fields = ['slug' => 'name'], $args = [])
{
    $taxonomy = MotoPress\Appointment\PostTypes\ServicePostType::CATEGORY_NAME;

    return mpa_get_terms($serviceId, $taxonomy, $fields, $args);
}

/**
 * @param WP_Term|int $term
 * @return int The total number of [unique] posts, including the posts of the
 *     child categories.
 *
 * @since 1.2
 */
function mpa_get_service_category_total_count($term)
{
    if (is_object($term)) {
        $termId = $term->term_id;
    } else {
        $termId = (int)$term;
    }

    // Get child IDs
    $categories = mpa_get_service_categories(0, 'term_id', ['hide_empty' => true, 'child_of' => $termId]);

    // Add parent ID
    $categories[] = $termId;

    // Get posts
    $posts = mpa_get_services([
        'fields'    => 'ids',
        'tax_query' => [
            [
                'taxonomy' => mpa_service()->getCategory(),
                'terms'    => $categories,
                'operator' => 'IN'
            ]
        ]
    ]);

    return count($posts);
}

/**
 * Extracts the minimum information needed for booking/appointment shortcodes.
 * For example:
 * <pre>
 *     $bookableServices = [
 *         134 => [
 *             'name'       => "Man's Haircut",
 *             'categories' => [
 *                 'barbershop' => 'Barbershop',
 *                 'haircuts'   => 'Haircuts'
 *             ],
 *             'employees'  => [
 *                 125 => [
 *                     'name'      => 'John Doe',
 *                     'schedule'  => 131,
 *                     'locations' => [
 *                         129 => 'Barbershop'
 *                     ]
 *                 ]
 *             ]
 *         ],
 *         135 => [
 *             'name'       => "Woman's Haircut",
 *             'categories' => [
 *                 'uncategorized' => 'Uncategorized'
 *             ],
 *             'employees'  => [
 *                 127 => [
 *                     'name'      => 'Mary Doe',
 *                     'schedule'  => 133,
 *                     'locations' => [
 *                         130 => 'Hairdressing Salon'
 *                     ]
 *                 ]
 *             ]
 *         ]
 *     ];
 * </pre>
 *
 * @return array
 *
 * @since 1.0
 */
function mpa_extract_available_services()
{
    $services  = mpa_get_services(['fields' => 'all']); // Service[]

    $schedules = mpa_get_schedules(['fields' => ['id' => 'employeeId']]);
    $schedules = array_flip($schedules); // [Employee ID => Schedule ID]

    $locations = []; // [Employee ID => [Location ID => Location name]],
                     // we'll fill it in the process (some sort of cache)

    // Filter services without employees; or services, which employees doesn't
    // have a schedule or working locations
    $bookableServices = [];

    foreach ($services as $service) {
        $serviceEmployees = [];

        // Add employees and their data
        $employeesWithSchedule = array_intersect($service->employeeIds, array_keys($schedules));

        foreach ($employeesWithSchedule as $employeeId) {
            // Check employee and schedule
            $employee = mpa_get_employee($employeeId);
            $schedule = mpa_get_schedule($schedules[$employeeId]);

            if (is_null($employee) || is_null($schedule)) {
                continue;
            }

            // Get a list of locations
            if (isset($locations[$employeeId])) {
                $employeeLocations = $locations[$employeeId];
            } else {
                $scheduleLocations = $schedule->getLocationIds(); // int[]

                if (!empty($scheduleLocations)) {
                    $employeeLocations = mpa_get_locations([
                        'post__in' => $scheduleLocations,
                        'fields'   => ['id' => 'name']
                    ]);
                } else {
                    $employeeLocations = [];
                }

                $locations[$employeeId] = $employeeLocations;
            }

            // Filter employees without locations
            if (empty($employeeLocations)) {
                continue;
            }

            // Fill employee data
            $serviceEmployees[$employeeId] = [
                'name'      => $employee->name,
                'schedule'  => $schedule->id,
                'locations' => $employeeLocations
            ];
        } // For each employee with schedule

        // Skip services without employees
        if (!empty($serviceEmployees)) {
            $categories = mpa_get_service_categories($service->id);

            if (!empty($categories)) {
                ksort($categories);
            } else {
                $categories['uncategorized'] = esc_html__('Uncategorized'); // WordPress core text
            }

            $bookableServices[$service->id] = [
                'name'       => $service->title,
                'categories' => $categories,
                'employees'  => $serviceEmployees
            ];
        }
    } // For each service

    return $bookableServices;
}

/**
 * @return Entities\AbstractEntity|null
 *
 * @since 1.2
 */
function mpa_get_current_entity()
{
	return mpa_get_entity();
}

/**
 * @param int $postId Optional. Current post by default.
 * @return Entities\AbstractEntity|null
 *
 * @since 1.2
 */
function mpa_get_entity($postId = 0)
{
	if ($postId == 0) {
		$postId = get_the_ID();
	}

    $postType = get_post_type($postId);

    $repository = mpapp()->repositories()->getByPostType($postType);

    if (!is_null($repository)) {
        return $repository->findById($postId);
    } else {
        return null;
    }
}
