<?php

namespace MotoPress\Appointment\Metaboxes;

use MotoPress\Appointment\Fields\AbstractField;
use MotoPress\Appointment\Fields\FieldsFactory;

if (!defined('ABSPATH')) {
    exit;
}

/**
 * @since 1.0
 */
abstract class FieldsMetabox extends AbstractMetabox
{
    /**
     * @var AbstractField[] [Prefixed field name (postmeta name) => Field object]
     *
     * @see FieldsMetabox::loadMetabox()
     *
     * @since 1.0
     */
    protected $fields = [];

    // abstract protected function theName();

    /**
     * @return array [Unprefixed field name => Field args]
     *
     * @since 1.0
     */
    abstract protected function theFields();

    /**
     * @since 1.0
     */
    protected function loadMetabox()
    {
        $fields = $this->theFields();
        /** @since 1.0 */
        $fields = apply_filters("{$this->id}_fields", $fields); // 'mpa_service_settings_metabox_fields'

        // Filter translatable fields
        if (mpa_is_translation_page($this->postType)) {
            $fields = FieldsFactory::filterTranslatable($fields);
        }

        $postId = get_the_ID(); // False on Add New post page

        // Instantiate the fields
        foreach ($fields as $name => $args) {
            if (isset($args['visibility']) && $args['visibility'] == 'public') {
                $metaName = mpa_prefix($name);
            } else {
                $metaName = mpa_prefix($name, 'private');
            }

            $metaValue = null;

            // Get the value
            if ($postId !== false) {
                $metaValues = get_post_meta($postId, $metaName);

                if (count($metaValues) == 1) {
                    $metaValue = reset($metaValues);
                } else if (count($metaValues) > 1) {
                    $metaValue = $metaValues;
                }
            }

            // Create the field
            $field = FieldsFactory::createField($metaName, $args, $metaValue);

            $this->fields[$metaName] = $field;
        }

        // No fields - no metabox (for example, on translation page)
        if (empty($this->fields)) {
            remove_action("mpa_register_{$this->entityType}_metaboxes", [$this, 'register'], 16);
        }
    }

    /**
     * @return string
     *
     * @since 1.0
     */
    protected function renderSide()
    {
        $output = '<div class="mpa-side-metabox">';

        foreach ($this->fields as $field) {
            $wrapperClass = 'mpa-meta-field';

            if ($field->getType() == 'hidden') {
                $wrapperClass .= ' mpa-hide';
            }

            $output .= '<div class="' . $wrapperClass . '">';
                if ($field->hasLabel()) {
                    $output .= '<div>' . $field->renderLabel() . '</div>';
                }

                $output .= $field->renderBody();
            $output .= '</div>';
        }

        $output .= '</div>';

        return $output;
    }

    /**
     * @return string
     *
     * @since 1.0
     */
    protected function renderRegular()
    {
        return mpa_render_template('private/fields/form-table.php', ['fields' => $this->fields]);
    }

    /**
     * @param int $postId
     * @param \WP_Post $post
     * @return array [Postmeta name => [add, update, delete]]
     *
     * @since 1.0
     */
    protected function parseValues($postId, $post)
    {
        $request = $_POST;

        $values = ['add' => [], 'update' => [], 'delete' => []];

        foreach ($this->fields as $metaName => $field) {
            if (!isset($request[$metaName])) {
                continue;
            }

            $rawValue = $request[$metaName];

            // Prepare single value
            if ($field->isSingle()) {
                $field->setValue($rawValue, 'validate');

                // Save all values, not just the new values
                $values['update'][$metaName] = $field->getValue('save');

            // Prepare multiple values
            } else {
                $oldValues = $field->getValue('save');

                $field->setValue($rawValue, 'validate');
                $newValues = $field->getValue('save');

                // Get rid of empty values - ''. If we pass value '' to
                // delete_post_meta() then it will delete all postmetas
                $addValues    = array_filter(array_diff($newValues, $oldValues));
                $deleteValues = array_filter(array_diff($oldValues, $newValues));

                if (!empty($addValues)) {
                    $values['add'][$metaName] = $addValues;
                }

                if (!empty($deleteValues)) {
                    $values['delete'][$metaName] = $deleteValues;
                }
            }
        } // For each field

        return $values;
    }

    /**
     * @param array $values [Postmeta name => [add, update, delete]]
     * @param int $postId
     * @param \WP_Post $post
     *
     * @since 1.0
     */
    protected function saveValues($values, $postId, $post)
    {
        // $metaValues is an array of values to add
        foreach ($values['add'] as $metaName => $metaValues) {
            mpa_add_post_metas($postId, $metaName, $metaValues);
        }

        // $metaValue is singular
        foreach ($values['update'] as $metaName => $metaValue) {
            update_post_meta($postId, $metaName, $metaValue);
        }

        // $metaValues is an array of values to delete
        foreach ($values['delete'] as $metaName => $metaValues) {
            mpa_delete_post_metas($postId, $metaName, $metaValues);
        }
    }

    // abstract public function getLabel();
}
