<?php

namespace MotoPress\Appointment\Metaboxes;

if (!defined('ABSPATH')) {
    exit;
}

/**
 * @since 1.0
 */
class SubmitMetabox extends AbstractMetabox
{
    /**
     * @var \MotoPress\Appointment\AdminPages\Edit\EditPostPage|null
     *
     * @since 1.0
     */
    protected $editPage = null;

    /**
     * @var \MotoPress\Appointment\PostTypes\Statuses\AbstractPostStatuses|null
     *
     * @since 1.0
     */
    protected $statuses = null;

    /**
     * @param string $postType
     * @param string $context Optional. 'side', 'normal' or 'advanced'. 'normal'
     *     by default.
     * @param string $priority Optional. 'low', 'default' or 'high'. 'default'
     *     by default.
     *
     * @since 1.0
     */
    public function __construct($postType, $context = 'side', $priority = 'default')
    {
        parent::__construct($postType, $context, $priority);

        $this->id = $this->name; // 'submitdiv'
    }

    /**
     * @return string
     *
     * @since 1.0
     */
    protected function theName()
    {
        return 'submitdiv';
    }

    /**
     * @since 1.0
     */
    protected function loadMetabox()
    {
        // Get edit page class
        $this->editPage = mpapp()->pages()->getEditPage($this->postType);

        // Get post type statuses
        $postType = mpapp()->postTypes()->getPostType($this->postType);

        if (!is_null($postType) && property_exists($postType, 'statuses')) {
            $this->statuses = $postType->statuses();
        }
    }

    /**
     * @access protected
     *
     * @since 1.0
     */
    public function register()
    {
        remove_meta_box('submitdiv', $this->postType, 'side');

        parent::register();
    }

    /**
     * @return string
     *
     * @since 1.0
     */
    protected function renderRegular()
    {
        // Not intended for 'normal' or 'advanced' contexts
        return $this->renderSide();
    }

    /**
     * @return string
     *
     * @since 1.0
     *
     * @todo Add filter "{$postType}_statuses".
     */
    protected function renderSide()
    {
        // Get current post
        $postId = get_the_ID();
        $post   = $postId !== false ? get_post($postId) : null;

        if (is_null($post)) {
            $post = new \WP_Post(new \stdClass());

            $post->ID = 0;
            $post->post_type = $this->postType;
        }

        // Is Add New page?
        $isNew = !is_null($this->editPage) && $this->editPage->isCurrentAddNewPage();

        // Get available statuses
        $statuses = !is_null($this->statuses) ? $this->statuses->getManualStatuses() : [];

        // Render metabox
        return mpa_render_template('private/metaboxes/custom-submit-metabox.php', [
            'post'     => $post,
            'is_new'   => $isNew,
            'statuses' => $statuses
        ]);
    }

    /**
     * @param int $postId
     * @param \WP_Post $post
     * @return array
     *
     * @since 1.0
     */
    protected function parseValues($postId, $post)
    {
        if (!empty($_POST['mpa_post_status'])) {
            $postStatus = sanitize_text_field( wp_unslash( $_POST['mpa_post_status'] ) );

            if (!is_null($this->statuses) && $this->statuses->hasStatus($postStatus)) {
                return ['post_status' => $postStatus];
            }
        }

        return [];
    }

    /**
     * @param array $values
     * @param int $postId
     * @param \WP_Post $post
     *
     * @since 1.0
     *
     * @todo Fix the double saving (original update + update of the post here).
     * @todo Test with WPML: sometimes it's impossible to add translations after
     *     the wp_update_post().
     */
    protected function saveValues($values, $postId, $post)
    {
        // Update post status
        if (isset($values['post_status'])) {
            wp_update_post(['ID' => $postId, 'post_status' => $values['post_status']]);
        }
    }

    /**
     * @return string
     *
     * @since 1.0
     */
    public function getLabel()
    {
        $postType  = mpapp()->postTypes()->getPostType($this->postType);
        $typeLabel = !is_null($postType) ? $postType->getSingularLabel() : esc_html_x('Post', 'Post type', 'motopress-appointment');

        // Translators: %s: The post type name, like "Service".
        return sprintf(esc_html__('Update %s', 'motopress-appointment'), $typeLabel);
    }
}
