<?php

namespace MotoPress\Appointment\Plugin;

if (!defined('ABSPATH')) {
    exit;
}

/**
 * @since 1.0
 */
class Assets
{
    /**
     * @var array [Handle => [$file, $dependencies = [], $version = '', $inFooter = true]]
     *
     * @since 1.0
     */
    protected $scripts = [];

    /**
     * @var string[] Script handles that require media scripts.
     *
     * @since 1.2.1
     */
    protected $mediaScripts = [];

    /**
     * @var array [Handle => [$file, $dependencies = [], $version = '', $media = 'all']]
     *
     * @since 1.0
     */
    protected $styles = [];

    /**
     * @since 1.0
     */
    public function __construct()
    {
        $this->addActions();
    }

    /**
     * @since 1.0
     */
    protected function addActions()
    {
        add_action('init', [$this, 'initAssets'], 15);
        add_action('init', [$this, 'registerAssets'], 15);
    }

    /**
     * @access protected
     *
     * @since 1.0
     */
    public function initAssets()
    {
        if (is_admin()) {
            $this->scripts = mpapp()->bundles()->assets()->getAdminScripts();
            $this->styles  = mpapp()->bundles()->assets()->getAdminStyles();
        } else {
            $this->scripts = mpapp()->bundles()->assets()->getPublicScripts();
            $this->styles  = mpapp()->bundles()->assets()->getPublicStyles();
        }

        $this->initLocalizations();
    }

    /**
     * @since 1.2.1
     */
    protected function initLocalizations()
    {
        $currentLanguage = mpapp()->i18n()->getCurrentLanguage();

        // Add Flatpickr localization
        $this->addFlatpickrLocalization($currentLanguage);
    }

    /**
     * @param string $language
     *
     * @since 1.2.1
     */
    protected function addFlatpickrLocalization($language)
    {
        /**
         * @param string $language
         *
         * @since 1.2.1
         */
        $language = apply_filters('mpa_flatpickr_l10n', $language);

        if (!mpa_is_flatpickr_l10n($language)) {
            return;
        }

        // Add l10n script
        $version = \MotoPress\Appointment\FLATPICKR_VERSION;
        $l10nUrl = mpa_filter_asset("vendors/flatpickr-{$version}/dist/l10n/{$language}.js");

        $this->scripts['mpa-flatpickr-l10n'] = [$l10nUrl, [], $version];

        // Load it with Flatpickr
        $scriptsWithflatpickr = $this->getDependentScripts('mpa-flatpickr');

        foreach ($scriptsWithflatpickr as $handle) {
            $this->addJsDependency($handle, 'mpa-flatpickr-l10n');
        }
    }

    /**
     * @access protected
     *
     * @since 1.0
     */
    public function registerAssets()
    {
        // Register scripts
        foreach ($this->scripts as $handle => $args) {
            list($file, $dependencies, $version, $inFooter) = $args + ['', [], '', true];

            $this->registerScript($handle, $file, $dependencies, $version, $inFooter);
        }

        // Register styles
        foreach ($this->styles as $handle => $args) {
            list($file, $dependencies, $version, $media) = $args + ['', [], '', 'all'];

            $this->registerStyle($handle, $file, $dependencies, $version, $media);
        }
    }

    /**
     * @param string $handle
     * @param string $file Absolute path to the file.
     * @param array $dependencies Optional.
     * @param string $version Optional. Current version of the plugin by default.
     * @param bool $inFooter Optional. True by default.
     * @return self
     *
     * @since 1.0
     */
    public function registerScript($handle, $file, $dependencies = [], $version = '', $inFooter = true)
    {
        if (!$version) {
            $version = mpa_version();
        }

        // Check "wp-media" dependency
        if (in_array('wp-media', $dependencies)) {
            // Remove "wp-media" from dependencies
            mpa_array_remove($dependencies, 'wp-media');

            // Save script handle
            $this->mediaScripts[] = $handle;
        }

        wp_register_script($handle, $file, $dependencies, $version, $inFooter);

        return $this;
    }

    /**
     * @param string $handle
     * @param string $file Absolute path to the file.
     * @param array $dependencies Optional.
     * @param string $version Optional. Current version of the plugin by default.
     * @param string $media Optional. 'all' by default.
     * @return self
     *
     * @since 1.0
     */
    public function registerStyle($handle, $file, $dependencies = [], $version = '', $media = 'all')
    {
        if (empty($version)) {
            $version = mpa_version();
        }

        wp_register_style($handle, $file, $dependencies, $version, $media);

        return $this;
    }

    /**
     * @param string $handle
     * @param string|callable $dependency New dependency or callback to filter
     *     dependencies.
     *
     * @since 1.0
     */
    public function addJsDependency($handle, $dependency)
    {
        $this->addDependencyTo($this->scripts, $handle, $dependency);
    }

    /**
     * @param string $handle
     * @param string|callable $dependency New dependency or callback to filter
     *     dependencies.
     *
     * @since 1.0
     */
    public function addCssDependency($handle, $dependency)
    {
        $this->addDependencyTo($this->styles, $handle, $dependency);
    }

    /**
     * @param array $assets
     * @param string $handle
     * @param string|callable $dependency
     *
     * @since 1.0
     */
    protected function addDependencyTo(&$assets, $handle, $dependency)
    {
        if (!array_key_exists($handle, $assets)) {
            return;
        }

        $currentDependencies = isset($assets[$handle][1]) ? $assets[$handle][1] : [];

        if (is_callable($dependency)) {
            $assets[$handle][1] = $dependency($currentDependencies);
        } else if (!in_array($dependency, $currentDependencies)) {
            $assets[$handle][1][] = $dependency;
        }
    }

    /**
     * @param string $dependency
     *
     * @since 1.2.1
     */
    public function getDependentScripts($dependency)
    {
        return $this->getDependentsOf($this->scripts, $dependency);
    }

    /**
     * @param string $dependency
     *
     * @since 1.2.1
     */
    public function getDependentStyles($dependency)
    {
        return $this->getDependentsOf($this->styles, $dependency);
    }

    /**
     * @param array $assets
     * @param string $dependency
     *
     * @since 1.2.1
     */
    protected function getDependentsOf(&$assets, $dependency)
    {
        $handles = [];

        foreach ($assets as $handle => $args) {
            if (isset($args[1]) && in_array($dependency, $args[1])) {
                $handles[] = $handle;
            }
        }

        return $handles;
    }

    /**
     * @param string[] $handles
     * @return self
     *
     * @since 1.0
     */
    public function enqueueScript(...$handles)
    {
        wp_enqueue_script($handles);

        // Maybe enqueue media scripts
        $this->enqueueMedia(...$handles);

        // Tell WordPress that scripts contain translations
        $this->enqueueTranslations(...$handles);

        return $this;
    }

    /**
     * @param string[] $handles
     * @return self
     *
     * @since 1.0
     */
    public function enqueueStyle(...$handles)
    {
        wp_enqueue_style($handles);

        return $this;
    }

    /**
     * @param string[] $handles
     * @return self
     *
     * @since 1.2.1
     */
    public function enqueueMedia(...$handles)
    {
        if (empty($handles)) {
            // Example:
            //     mpa_assets()->enqueueMedia();
            wp_enqueue_media();

        } else {
            // Example:
            //     mpa_assets()->enqueueScripts(...);
            $mediaScripts = array_intersect($handles, $this->mediaScripts);

            if (!empty($mediaScripts)) {
                wp_enqueue_media();
            }
        }

        return $this;
    }

    /**
     * @param string[] $handles
     * @return self
     *
     * @since 1.2.1
     */
    public function enqueueTranslations(...$handles)
    {
        foreach ($handles as $handle) {
            $dependencies = [];

            if (array_key_exists($handle, $this->scripts) && !empty($this->scripts[$handle][1])) {
                $dependencies = $this->scripts[$handle][1];
            }

            if (in_array('wp-i18n', $dependencies)) {
                wp_set_script_translations($handle, 'motopress-appointment', mpa_languages_dir('absolute'));
            }
        }

        return $this;
    }

    /**
     * @param string[] $handles
     * @return self
     *
     * @since 1.0
     */
    public function enqueueBundle(...$handles)
    {
        $this->enqueueScript(...$handles);
        $this->enqueueStyle(...$handles);

        return $this;
    }
}
