<?php

namespace MotoPress\Appointment\Plugin;

if (!defined('ABSPATH')) {
	exit;
}

/**
 * @since 1.4.0
 */
class Upgrader
{
	/** @since 1.4.0 */
	const OPTION_DB_VERSION = 'mpa_db_version';

	/**
	 * Database versions history option name.
	 *
	 * @since 1.4.0
	 */
	const OPTION_DB_HISTORY = 'mpa_db_history';

	/**
	 * All versions that need upgrade.
	 *
	 * @since 1.4.0
	 * @var array[] [Version => Upgrade function(-s)]
	 */
	protected $neededUpgrade = [
		// Use long versions, e.g. '1.4.0' instead of '1.4' (similar to the
		// constant Appointment\VERSION)
		'1.4.0' => 'upgradeEmailsForMultibooking',
	];

	/**
	 * Current plugin version.
	 *
	 * @since 1.4.0
	 * @var string
	 */
	protected $pluginVersion = '1.0.0';

	/**
	 * Current DB version.
	 *
	 * @since 1.4.0
	 * @var string
	 */
	protected $dbVersion = '1.0.0';

	/**
	 * Previous versions history.
	 *
	 * @since 1.4.0
	 * @var string Versions, separated with comma.
	 */
	protected $dbHistory = '';

	/**
	 * @since 1.4.0
	 */
	public function __construct()
	{
		$this->checkVersions();
		$this->addActions();
	}

	/**
	 * @since 1.4.0
	 */
	protected function checkVersions()
	{
		$this->pluginVersion = mpa_version();
		$this->dbVersion	 = get_option(self::OPTION_DB_VERSION, $this->dbVersion);
		$this->dbHistory	 = get_option(self::OPTION_DB_HISTORY, $this->dbHistory);
	}

	/**
	 * @since 1.4.0
	 */
	protected function addActions()
	{
		add_action('init', [$this, 'maybeUpgrade']);
	}

	/**
	 * @since 1.4.0
	 * @access protected
	 */
	public function maybeUpgrade()
	{
		if (version_compare($this->dbVersion, $this->pluginVersion, '=')) {
			// No upgrade required
			return;
		}

		$this->upgrade();
	}

	/**
	 * @since 1.4.0
	 */
	protected function upgrade()
	{
		mpa_set_time_limit(0);

		foreach ($this->neededUpgrade as $upgradeVersion => $callbacks) {
			if (version_compare($this->dbVersion, $upgradeVersion, '<')) {
				$this->upgradeTo($upgradeVersion, $callbacks);
			}
		}

		if (version_compare($this->dbVersion, $this->pluginVersion, '!=')) {
			$this->setVersion($this->pluginVersion);
		}
	}

	/**
	 * @since 1.4.0
	 *
	 * @param string $version
	 * @param callable|callable[]
	 */
	protected function upgradeTo($version, $callbacks)
	{
		foreach ((array)$callbacks as $callback) {
			if (function_exists($callback)) {
				$callback();
			} else {
				$this->$callback();
			}
		}

		$this->setVersion($version);
	}

	/**
	 * @since 1.4.0
	 *
	 * @param string $version
	 */
	protected function setVersion($version)
	{
		$this->addHistory($this->dbVersion);
		$this->dbVersion = $version;

		update_option(self::OPTION_DB_VERSION, $this->dbVersion);
	}

	/**
	 * @since 1.4.0
	 *
	 * @param string $version
	 */
	protected function addHistory($version)
	{
		if (!empty($this->dbHistory)) {
			// Add separator
			$this->dbHistory .= ',';
		}

		$this->dbHistory .= $version;

		update_option(self::OPTION_DB_HISTORY, $this->dbHistory);
	}

	/**
	 * Removes:
	 * 1) admin and customer emails (only subjects, headers and messages);
	 * 2) template parts.
	 *
	 * @since 1.4.0
	 * @access protected
	 * @global \wpdb $wpdb
	 */
	public function upgradeEmailsForMultibooking()
	{
		global $wpdb;

		$sql = "DELETE FROM {$wpdb->options}"
			. " WHERE `option_name` LIKE 'mpa_%_email_subject'"
				. " OR `option_name` LIKE 'mpa_%_email_header'"
				. " OR `option_name` LIKE 'mpa_%_email_message'"
				. " OR `option_name` LIKE 'mpa_%_reservation_details'";

		$wpdb->query($sql);
	}
}
