<?php

namespace MotoPress\Appointment\Registries;

use MotoPress\Appointment\Emails\Admin;
use MotoPress\Appointment\Emails\Customer;
use MotoPress\Appointment\Emails\Tags;
use MotoPress\Appointment\Emails\AbstractEmail;
use MotoPress\Appointment\Emails\EmailTags;

if (!defined('ABSPATH')) {
    exit;
}

/**
 * @since 1.1.0
 */
class EmailsRegistry
{
    /**
     * @var AbstractEmail[]
     *
     * @since 1.1.0
     */
    protected $emails = [];

    /**
     * @return Admin\AdminNewBookingEmail
     *
     * @since 1.1.0
     */
    public function adminNewBooking()
    {
        if (!isset($this->emails['adminNewBooking'])) {
            $this->emails['adminNewBooking'] = new Admin\AdminNewBookingEmail($this->getAdminEmailTags());
        }

        return $this->emails['adminNewBooking'];
    }

    /**
     * @return Admin\AdminPendingBookingEmail
     *
     * @since 1.1.0
     */
    public function adminPendingBooking()
    {
        if (!isset($this->emails['adminPendingBooking'])) {
            $this->emails['adminPendingBooking'] = new Admin\AdminPendingBookingEmail($this->getAdminEmailTags());
        }

        return $this->emails['adminPendingBooking'];
    }

    /**
     * @return Customer\CustomerNewBookingEmail
     *
     * @since 1.1.0
     */
    public function customerNewBooking()
    {
        if (!isset($this->emails['customerNewBooking'])) {
            $this->emails['customerNewBooking'] = new Customer\CustomerNewBookingEmail($this->getCustomerEmailTags());
        }

        return $this->emails['customerNewBooking'];
    }

    /**
     * @return Customer\CustomerPendingBookingEmail
     *
     * @since 1.1.0
     */
    public function customerPendingBooking()
    {
        if (!isset($this->emails['customerPendingBooking'])) {
            $this->emails['customerPendingBooking'] = new Customer\CustomerPendingBookingEmail($this->getCustomerEmailTags());
        }

        return $this->emails['customerPendingBooking'];
    }

    /**
     * @return Customer\CustomerApprovedBookingEmail
     *
     * @since 1.1.0
     */
    public function customerApprovedBooking()
    {
        if (!isset($this->emails['customerApprovedBooking'])) {
            $this->emails['customerApprovedBooking'] = new Customer\CustomerApprovedBookingEmail($this->getCustomerEmailTags());
        }

        return $this->emails['customerApprovedBooking'];
    }

    /**
     * @return Customer\CustomerCancelledBookingEmail
     *
     * @since 1.1.0
     */
    public function customerCancelledBooking()
    {
        if (!isset($this->emails['customerCancelledBooking'])) {
            $this->emails['customerCancelledBooking'] = new Customer\CustomerCancelledBookingEmail($this->getCustomerEmailTags());
        }

        return $this->emails['customerCancelledBooking'];
    }

    /**
     * @return AbstractEmail[]
     *
     * @since 1.1.0
     */
    public function getAdminEmails()
    {
        $adminEmails = [
            $this->adminNewBooking(),
            $this->adminPendingBooking()
        ];

        $adminEmails = $this->mapByName($adminEmails);

        /** @since 1.1.0 */
        $adminEmails = apply_filters('mpa_admin_emails', $adminEmails);

        return $adminEmails;
    }

    /**
     * @return AbstractEmail[]
     *
     * @since 1.1.0
     */
    public function getCustomerEmails()
    {
        $customerEmails = [
            $this->customerNewBooking(),
            $this->customerPendingBooking(),
            $this->customerApprovedBooking(),
            $this->customerCancelledBooking()
        ];

        $customerEmails = $this->mapByName($customerEmails);

        /** @since 1.1.0 */
        $customerEmails = apply_filters('mpa_customer_emails', $customerEmails);

        return $customerEmails;
    }

    /**
     * @return AbstractEmail[]
     *
     * @since 1.1.0
     */
    public function getEmails()
    {
        return array_merge($this->getAdminEmails(), $this->getCustomerEmails());
    }

    /**
     * @param string $name
     * @return AbstractEmail|null
     *
     * @since 1.1.0
     */
    public function getEmailByName($name)              // 'admin_pending_booking_email'
    {
        $entryId = mpa_str_to_method_name($name);      // 'adminPendingBookingEmail'
        $entryId = str_replace('Email', '', $entryId); // 'adminPendingBooking'

        // Get email
        if (method_exists($this, $entryId)) {
            return $this->$entryId();
        } else {
            return null;
        }
    }

    /**
     * @return EmailTags
     *
     * @since 1.1.0
     */
    public function getAdminEmailTags()
    {
        $adminEmailTags = [
            new Tags\GeneralTags(),
            new Tags\BookingTags(),
            new Tags\AdminReservationTags(),
            new Tags\ServiceTags(),
            new Tags\EmployeeTags(),
            new Tags\LocationTags(),
        ];

        /** @since 1.1.0 */
        $adminEmailTags = apply_filters('mpa_admin_email_tags', $adminEmailTags);

        return new EmailTags($adminEmailTags);
    }

    /**
     * @return EmailTags
     *
     * @since 1.1.0
     */
    public function getCustomerEmailTags()
    {
        $customerEmailTags = [
            new Tags\GeneralTags(),
            new Tags\BookingTags(),
            new Tags\CustomerReservationTags(),
            new Tags\ServiceTags(),
            new Tags\EmployeeTags(),
            new Tags\LocationTags(),
        ];

        /** @since 1.1.0 */
        $customerEmailTags = apply_filters('mpa_customer_email_tags', $customerEmailTags);

        return new EmailTags($customerEmailTags);
    }

    /**
     * @return EmailTags
     *
     * @since 1.1.0
     */
    public function getTemplatePartTags()
    {
        $templatePartTags = [
            new Tags\ReservationTags(),
            new Tags\ServiceTags(),
            new Tags\EmployeeTags(),
            new Tags\LocationTags()
        ];

        /** @since 1.1.0 */
        $templatePartTags = apply_filters('mpa_email_template_part_tags', $templatePartTags);

        return new EmailTags($templatePartTags);
    }

    /**
     * @param AbstractEmail[] $entries [Entry ID => AbstractEmail]
     * @return AbstractEmail[] [Entry name => AbstractEmail]
     *
     * @since 1.1.0
     */
    protected function mapByName($entries)
    {
        $mapped = [];

        foreach ($entries as $entry) {
            $mapped[$entry->getName()] = $entry;
        }

        return $mapped;
    }
}
