<?php

namespace MotoPress\Appointment\Repositories;

use MotoPress\Appointment\Entities\Reservation;
use MotoPress\Appointment\Structures\TimePeriod;

if (!defined('ABSPATH')) {
    exit;
}

/**
 * @since 1.0
 *
 * @see Reservation
 */
class ReservationRepository extends AbstractRepository
{
    /**
     * @since 1.0
     */
    protected function addActions()
    {
        add_filter("{$this->postType}_repository_get_posts_query_args", [$this, 'filterArgs'], 5);
    }

    /**
     * @return array
     *
     * @since 1.0
     */
    protected function entitySchema()
    {
        return [
            'post'     => ['ID', 'post_parent'],
            'postmeta' => [
                '_mpa_price'        => true,
                '_mpa_date'         => true,
                '_mpa_service_time' => true,
                '_mpa_buffer_time'  => true,
                '_mpa_service'      => true,
                '_mpa_employee'     => true,
                '_mpa_location'     => true,
                '_mpa_capacity'     => true
            ]
        ];
    }

    /**
     * @param array $postData
     * @return Employee
     *
     * @since 1.0
     */
    protected function mapPostDataToEntity($postData)
    {
        $id = (int)$postData['ID'];

        $date        = mpa_parse_date($postData['date']);
        $serviceTime = new TimePeriod($postData['service_time']);
        $bufferTime  = new TimePeriod($postData['buffer_time']);

        $serviceTime->setDate($date);
        $bufferTime->setDate($date);

        $fields = [
            'bookingId'   => (int)$postData['post_parent'],
            'price'       => (float)$postData['price'],
            'date'        => $date,
            'serviceTime' => $serviceTime,
            'bufferTime'  => $bufferTime,
            'serviceId'   => (int)$postData['service'],
            'employeeId'  => (int)$postData['employee'],
            'locationId'  => (int)$postData['location'],
            'capacity'    => (int)$postData['capacity']
        ];

        return new Reservation($id, $fields);
    }

    /**
     * @param int $bookingId
     * @return Reservation[]
     *
     * @since 1.0
     */
    public function findAllByBooking($bookingId)
    {
        return $this->findAll(['post_parent' => $bookingId]);
    }

    /**
     * @param int $serviceId
     * @param array $args Optional.
     *     @param \DateTime|string $args['from_date']
     *     @param \DateTime|string $args['to_date']
     *     @param string|array     $args['fields']
     * @return array
     *
     * @since 1.0
     */
    public function findAllByService($serviceId, $args = [])
    {
        $args['service_id'] = $serviceId;

        return $this->findAll($args);
    }

    /**
     * @param array $args
     * @return array
     *
     * @access protected
     *
     * @since 1.0
     */
    public function filterArgs($args)
    {
        $metaQuery = [];

        // Service ID, employee ID, location ID
        $idFields = [
            '_mpa_service'  => 'service_id',
            '_mpa_employee' => 'employee_id',
            '_mpa_location' => 'location_id'
        ];

        foreach ($idFields as $postmeta => $name) {
            if (isset($args[$name])) {
                $value = $args[$name];

                $metaQuery[] = [
                    'key'     => $postmeta,
                    'value'   => $value,
                    'compare' => is_array($value) ? 'IN' : '='
                ];

                unset($args[$name]);
            }
        }

        // Start date
        if (isset($args['from_date'])) {
            $fromDate = is_string($args['from_date']) ? $args['from_date'] : mpa_format_date($args['from_date'], 'internal');

            $metaQuery[] = [
                'key'     => '_mpa_date',
                'value'   => $fromDate,
                'compare' => '>='
            ];

            unset($args['from_date']);
        }

        // End date
        if (isset($args['to_date'])) {
            $toDate = is_string($args['to_date']) ? $args['to_date'] : mpa_format_date($args['to_date'], 'internal');

            $metaQuery[] = [
                'key'     => '_mpa_date',
                'value'   => $toDate,
                'compare' => '<='
            ];

            unset($args['to_date']);
        }

        // Merge meta query
        if (!empty($metaQuery)) {
            if (!isset($args['meta_query'])) {
                $args['meta_query'] = $metaQuery;
            } else {
                $args['meta_query'][] = $metaQuery;
            }
        }

        return $args;
    }
}
