<?php

namespace MotoPress\Appointment\Repositories;

use MotoPress\Appointment\Entities\Service;
use MotoPress\Appointment\PostTypes\ServicePostType;
use MotoPress\Appointment\Structures\Service\ServiceVariation;
use MotoPress\Appointment\Structures\Service\ServiceVariations;

if (!defined('ABSPATH')) {
	exit;
}

/**
 * @since 1.0
 * @see Service
 */
class ServiceRepository extends AbstractRepository
{
	/**
	 * @since 1.0
	 *
	 * @return array
	 */
	protected function entitySchema()
	{
		return [
			'post'     => ['ID', 'post_title', 'post_content'],
			'postmeta' => [
				'_mpa_price'               => true,
				'_mpa_duration'            => true,
				'_mpa_buffer_time_before'  => true,
				'_mpa_buffer_time_after'   => true,
				'_mpa_time_before_booking' => true,
				'_mpa_min_capacity'        => true,
				'_mpa_max_capacity'        => true,
				'_mpa_allow_guests'        => true,
				'_mpa_multiply_price'      => true,
				'_mpa_employees'           => true,
				'_mpa_variations'          => true,
			]
		];
	}

	/**
	 * @since 1.0
	 *
	 * @param array $postData
	 * @return Service
	 *
	 * @todo Move upgrade code to plugin upgrader.
	 */
	protected function mapPostDataToEntity($postData)
	{
		$id = (int)$postData['ID'];

		// $minCapacity = $maxCapacity = 0 for existing services without
		// postmetas (upgrade 1.3 to 1.4), so we need to set the minimum values
		$minCapacity = max(1, (int)$postData['min_capacity']);
		$maxCapacity = max($minCapacity, (int)$postData['max_capacity']);

		$fieldsData = [
			'employeeIds'       => array_map('mpa_posint', (array)$postData['employees']),
			'title'             => $postData['post_title'],
			'description'       => $postData['post_content'],
			'price'             => (float)$postData['price'],
			'duration'          => (int)$postData['duration'],
			'bufferTimeBefore'  => (int)$postData['buffer_time_before'],
			'bufferTimeAfter'   => (int)$postData['buffer_time_after'],
			'timeBeforeBooking' => (int)$postData['time_before_booking'],
			'minCapacity'       => $minCapacity,
			'maxCapacity'       => $maxCapacity,
			'allowGuests'       => (bool)$postData['allow_guests'],
			'multiplyPrice'     => (bool)$postData['multiply_price'],
		];

		$fieldsData['variations'] = $this->buildVariations($postData, $fieldsData);

		return new Service($id, $fieldsData);
	}

	/**
	 * @since 1.3.1
	 *
	 * @param array $postData
	 * @param array $fieldsData
	 * @return ServiceVariations
	 *
	 * @todo Move upgrade code to plugin upgrader.
	 */
	protected function buildVariations($postData, $fieldsData)
	{
		if (!is_array($postData['variations'])) {
			return [];
		}

		$variations = [];

		foreach ($postData['variations'] as $variationData) {
			// Add optional fields (upgrade 1.3 to 1.4)
			$variationData += [
				'min_capacity' => 1,
				'max_capacity' => 1,
			];

			if ($variationData['price'] === '') {
				$variationData['price'] = $fieldsData['price'];
			}

			if ($variationData['min_capacity'] === '') {
				$variationData['min_capacity'] = $fieldsData['minCapacity'];
			}

			if ($variationData['max_capacity'] === '') {
				$variationData['max_capacity'] = $fieldsData['maxCapacity'];
			}

			$variationData['max_capacity'] = max($variationData['min_capacity'], $variationData['max_capacity']);

			// Add variation
			$employeeId = (int)$variationData['employee'];
			$variations[$employeeId] = new ServiceVariation($variationData);
		}

		return new ServiceVariations($variations);
	}

	/**
	 * @since 1.0
	 *
	 * @param array $args Optional. Additional arguments for function get_terms().
	 *     [] by default.
	 * @return array [Term ID => WP_Term]
	 */
	public function findCategories($args = [])
	{
		return $this->getCategories(ServicePostType::CATEGORY_NAME, $args);
	}

	/**
	 * @since 1.2
	 *
	 * @param int $employeeId
	 * @param array $args Optional.
	 * @return array
	 */
	public function findAllByEmployee($employeeId, $args = [])
	{
		$employeesMeta = mpa_prefix('employees', 'private');

		return $this->findAllByValueInMeta($employeesMeta, $employeeId, $args);
	}
}
