<?php

namespace MotoPress\Appointment\REST\Controllers;

use MotoPress\Appointment\Entities\Reservation;
use MotoPress\Appointment\PostTypes\Statuses\BookingStatuses;
use MotoPress\Appointment\Services\BookingService;
use WP_Error;
use WP_REST_Request;
use WP_REST_Response;
use WP_REST_Server;

if (!defined('ABSPATH')) {
    exit;
}

/**
 * @since 1.0
 */
class BookingsRestController extends AbstractRestController
{
    /**
     * @since 1.0
     */
    public function registerRoutes()
    {
        // '/motopress/appointment/v1/bookings'
        register_rest_route($this->getNamespace(), '/bookings', [
            'methods'             => WP_REST_Server::CREATABLE,
            'callback'            => [$this, 'createBooking'],
            'permission_callback' => '__return_true',
            'args'                => [
                'order'               => [
                    'default'             => []
                ]
            ]
        ]);

        // '/motopress/appointment/v1/bookings/reservations'
        register_rest_route($this->getNamespace(), '/bookings/reservations', [
            'methods'             => WP_REST_Server::READABLE,
            'callback'            => [$this, 'getReservations'],
            'permission_callback' => '__return_true',
            'args'                => [
                'service_id'          => [
                    'default'             => [],
                    'sanitize_callback'   => 'mpa_rest_sanitize_ids'
                ],
                'from_date'           => [
                    'default'             => '',
                    'sanitize_callback'   => 'mpa_rest_sanitize_date_string'
                ],
                'to_date'             => [
                    'default'             => '',
                    'sanitize_callback'   => 'mpa_rest_sanitize_date_string'
                ]
            ]
        ]);
    }

    /**
     * @param WP_REST_Request $request
     * @return WP_REST_Response|WP_Error
     *
     * @since 1.0
     */
    public function createBooking($request)
    {
        $bookingService = new BookingService();

        $order = $request->get_params('order');
        $error = $booking = $bookingService->createBooking($order);

        if (is_wp_error($booking)) {
            return rest_ensure_response($error);
        } else {
            $saved = (bool)$bookingService->saveBooking($booking);

            if ($saved) {
                if (mpapp()->settings()->getDefaultBookingStatus() == BookingStatuses::STATUS_CONFIRMED) {
                    $message = esc_html__('Your reservation is confirmed. Thank You!', 'motopress-appointment');
                } else {
                    $message = esc_html__('Your reservation request has been received. Thank you!', 'motopress-appointment');
                }
            } else {
                $message = esc_html__('Sorry! Failed to make a reservation at the moment.', 'motopress-appointment');
            }

            return rest_ensure_response(['message' => $message]);
        }
    }

    /**
     * @param WP_REST_Request $request
     * @return WP_REST_Response|WP_Error
     *
     * @since 1.0
     */
    public function getReservations($request)
    {
        $serviceId = $request->get_param('service_id');
        $fromDate  = $request->get_param('from_date');
        $toDate    = $request->get_param('to_date');

        // "Get all reservations" is not supported in the current version
        if (empty($serviceId)) {
            return mpa_rest_request_error(esc_html__('Invalid parameter: service ID is not set.', 'motopress-appointment'));
        } else if (empty($fromDate) || empty($toDate)) {
            return mpa_rest_request_error(esc_html__('Invalid parameter: date range is not set.', 'motopress-appointment'));
        }

        // Find reservations
        $args = [
            'from_date' => $fromDate,
            'to_date'   => $toDate
        ];

        $reservations = mpapp()->repositories()->reservation()->findAllByService($serviceId, $args);
        $reservations = array_map([$this, 'mapReservation'], $reservations);

        return rest_ensure_response($reservations);
    }

    /**
     * @param Reservation $reservation
     * @return array
     *
     * @access protected
     *
     * @since 1.0
     */
    public function mapReservation($reservation)
    {
        $entityData = [
            'id'          => $reservation->id,
            'bookingId'   => $reservation->bookingId,
            'price'       => $reservation->price,
            'date'        => mpa_format_date($reservation->date, 'internal'),
            'serviceTime' => $reservation->serviceTime->toString('internal'),
            'bufferTime'  => $reservation->bufferTime->toString('internal'),
            'serviceId'   => $reservation->serviceId,
            'employeeId'  => $reservation->employeeId,
            'locationId'  => $reservation->locationId,
            'capacity'    => $reservation->capacity
        ];

        return $entityData;
    }
}
