<?php

namespace MotoPress\Appointment\REST\Controllers;

use MotoPress\Appointment\Services\TimeSlotService;

use WP_Error;
use WP_REST_Request;
use WP_REST_Response;
use WP_REST_Server;

if (!defined('ABSPATH')) {
    exit;
}

/**
 * @since 1.2.1
 */
class CalendarRestController extends AbstractRestController
{
    /**
     * @since 1.2.1
     */
    public function registerRoutes()
    {
        // '/motopress/appointment/v1/calendar/time'
        register_rest_route($this->getNamespace(), '/calendar/time', [
            'methods'             => WP_REST_Server::READABLE,
            'callback'            => [$this, 'getTimeSlots'],
            'permission_callback' => '__return_true',
            'args'                => [
                // service_id, from_date and to_date are required
                'service_id'          => [
                    // Required
                    'default'             => 0,
                    'sanitize_callback'   => 'mpa_rest_sanitize_id'
                ],
                'employees'           => [
                    'default'             => [],
                    'sanitize_callback'   => 'mpa_rest_sanitize_ids'
                ],
                'locations'           => [
                    'default'             => [],
                    'sanitize_callback'   => 'mpa_rest_sanitize_ids'
                ],
                'from_date'           => [
                    // Required
                    'default'             => '',
                    'sanitize_callback'   => 'mpa_rest_sanitize_date_string'
                ],
                'to_date'             => [
                    // Required
                    'default'             => '',
                    'sanitize_callback'   => 'mpa_rest_sanitize_date_string'
                ],
                'format'              => [
                    'default'             => 'compact',
                    'sanitize_callback'   => 'sanitize_text_field'
                ],
				'exclude'             => [
					// Optional. Items to exclude in the format of the cart items.
					'default'             => [],
					'sanitize_callback'   => 'mpa_rest_sanitize_cart_items',
				],
                'skip_empty'          => [
                    // Only if format='compact'
                    'default'             => true,
                    'sanitize_callback'   => 'mpa_rest_sanitize_bool'
                ],
                'since_today'         => [
                    'default'             => true,
                    'sanitize_callback'   => 'mpa_rest_sanitize_bool'
                ]
            ]
        ]);
    }

    /**
     * @param WP_REST_Request $request
     * @return WP_REST_Response|WP_Error
     *
     * @since 1.2.1
     */
    public function getTimeSlots($request)
    {
        $serviceId  = $request->get_param('service_id');
        $employees  = $request->get_param('employees');
        $locations  = $request->get_param('locations');
        $fromDate   = $request->get_param('from_date');
        $toDate     = $request->get_param('to_date');
        $format     = $request->get_param('format');
        $exclude    = $request->get_param('exclude');
        $skipEmpty  = $request->get_param('skip_empty');
        $sinceToday = $request->get_param('since_today');

        // Check required fields
        if (!$serviceId) {
            return mpa_rest_request_error(esc_html__('Invalid parameter: service ID is not set.', 'motopress-appointment'));
        } else if (!$fromDate || !$toDate) {
            return mpa_rest_request_error(esc_html__('Invalid parameter: date range is not set.', 'motopress-appointment'));
        }

        $service = mpa_get_service($serviceId);

        if (is_null($service)) {
            return mpa_rest_request_error(sprintf(esc_html__('Invalid request: service #$d not found.', 'motopress-appointment'), $serviceId));
        }

        $timeSlotService = new TimeSlotService($service, ['employees' => $employees, 'locations' => $locations]);

        // Block reserved periods
        $timeSlotService->blockPeriod($fromDate, $toDate);

		// Block cart items
		$timeSlotService->blockCartItems($exclude);

        // Build time slots
        switch ($format) {
            case 'full':
                // ['Y-m-d' => [Time period string => Array of [Employee ID, Location ID]]]
                $timeSlots = $timeSlotService->getTimeSlotsForEntities($fromDate, $toDate, $sinceToday);
                break;

            case 'compact':
            default:
                $slotsArgs = [
                    'skip_empty'  => $skipEmpty,
                    'since_today' => $sinceToday
                ];

                // ['Y-m-d' => [Time period string => Time period]]
                $timeSlots = $timeSlotService->getAvailableTimeSlotsForPeriod($fromDate, $toDate, $slotsArgs);

				// ['Y-m-d' => [Time period string]]
				$timeSlots = array_map(
					function ($daySlots) { return array_keys($daySlots); },
					$timeSlots
				);
                break;
        }

        return rest_ensure_response($timeSlots);
    }
}
