<?php

namespace MotoPress\Appointment\REST\Controllers;

use MotoPress\Appointment\Entities\Service;
use WP_Error;
use WP_REST_Request;
use WP_REST_Response;
use WP_REST_Server;

if (!defined('ABSPATH')) {
    exit;
}

/**
 * @since 1.0
 */
class ServicesRestController extends AbstractRestController
{
    /**
     * @since 1.0
     */
    public function registerRoutes()
    {
        // '/motopress/appointment/v1/services'
        register_rest_route($this->getNamespace(), '/services', [
            'methods'             => WP_REST_Server::READABLE,
            'callback'            => [$this, 'getServices'],
            'permission_callback' => '__return_true',
            'args'                => [
                'id'                  => [
                    'default'             => 0,
                    'sanitize_callback'   => 'mpa_rest_sanitize_id'
                ]
            ]
        ]);

        // '/motopress/appointment/v1/services/available'
        register_rest_route($this->getNamespace(), '/services/available', [
            'methods'             => WP_REST_Server::READABLE,
            'callback'            => [$this, 'getAvailableServices'],
            'permission_callback' => '__return_true'
        ]);
    }

    /**
     * @param WP_REST_Request $request
     * @return WP_REST_Response|WP_Error
     *
     * @since 1.0
     */
    public function getServices($request)
    {
        // Get and check the input
        $serviceId = $request->get_param('id');

        if (!$serviceId) {
            // "Get all services" is not supported in the current version
            return mpa_rest_request_error(esc_html__('Invalid parameter: service ID is not set.', 'motopress-appointment'));
        }

        // Load service
        $service = mpa_get_service($serviceId);

        // Return the response
        if (!is_null($service)) {
            return rest_ensure_response($this->mapEntity($service));
        } else {
            return mpa_rest_request_error(sprintf(esc_html__('Invalid request: service #$d not found.', 'motopress-appointment'), $serviceId));
        }
    }

    /**
     * @param WP_REST_Request $request
     * @return WP_REST_Response|WP_Error
     *
     * @since 1.0
     */
    public function getAvailableServices($request)
    {
        return rest_ensure_response(mpa_extract_available_services());
    }

    /**
     * @param Service $entity
     * @return array
     *
     * @since 1.0
     */
    protected function mapEntity($entity)
    {
        $entityData = [
            'id'                => $entity->id,
            'name'              => $entity->title,
            'description'       => $entity->description,
            'price'             => $entity->price,
            'duration'          => $entity->duration,
            'bufferTimeBefore'  => $entity->bufferTimeBefore,
            'bufferTimeAfter'   => $entity->bufferTimeAfter,
            'timeBeforeBooking' => $entity->timeBeforeBooking,
            'minCapacity'       => $entity->minCapacity,
            'maxCapacity'       => $entity->maxCapacity,
			'allowGuests'       => $entity->allowGuests,
			'multiplyPrice'     => $entity->multiplyPrice,
            'employeeIds'       => $entity->employeeIds,
            'variations'        => $entity->variations->toArray('deep'),
            'image'             => mpa_get_post_attachment_image_url($entity->id, 'post-thumbnail'),
            'thumbnail'         => mpa_get_post_attachment_image_url($entity->id),
        ];

        return $entityData;
    }
}
