<?php

namespace MotoPress\Appointment\Services;

use MotoPress\Appointment\Entities;
use MotoPress\Appointment\Structures\TimePeriod;
use MotoPress\Appointment\PostTypes;
use RuntimeException;
use WP_Error;

if (!defined('ABSPATH')) {
    exit;
}

/**
 * @since 1.0
 */
class BookingService
{
    /**
     * @param array $order
     * @return Entities\Booking|WP_Error
     *
     * @since 1.0
     */
    public function createBooking($order)
    {
        if (!is_array($order)) {
            return new WP_Error('invalid_input', esc_html__('Unable to make a reservation. Your order seems to be empty.', 'motopress-appointment'));
        }

        // Create reservations
        $reservations = [];
        $totalPrice = 0.0;

        foreach ($order['items'] as $item) {
            $reservation = $this->createReservation($item);

            if (is_wp_error($reservation)) {
                return $reservation; // return WP_Error
            }

            $reservations[] = $reservation;

            $totalPrice += $reservation->price;
        }

        // Create customer
        $customer = $this->parseCustomer($order);

        // Create booking
        $booking = new Entities\Booking(0, [
            'totalPrice'   => $totalPrice,
            'reservations' => $reservations,
            'customer'     => $customer
        ]);

        return $booking;
    }

    /**
     * @param array $item Order item
     * @return Entities\Reservation|WP_Error
     *
     * @since 1.0
     */
    public function createReservation($item)
    {
        $reservation = null;

        try {
            $this->requireFields($item);

            // Required properties
            $service  = $this->parseService($item);
            $date     = $this->parseDate($item);
            $time     = $this->parseTime($item);

            $time->setDate($date);

            // Optional properties
            $employee = $this->parseEmployee($item, $service, $date);
            $location = $this->parseLocation($item, $employee, $date);

            // Prepare other properties
			$capacity = $this->parseCapacity($item, $service, $employee);

			$price = $service->getPrice($employee->id, $capacity);

			$bufferTime = mpa_add_buffer_time(clone $time, $service);

            $reservationFields = [
                'price'       => $price,
                'date'        => $date,
                'serviceTime' => $time,
                'bufferTime'  => $bufferTime,
                'serviceId'   => $service->id,
                'employeeId'  => $employee->id,
				'locationId'  => $location->id,
				'capacity'    => $capacity,
            ];

            $reservation = new Entities\Reservation(0, $reservationFields);

        } catch (RuntimeException $error) {
            return new WP_Error('invalid_input', $error->getMessage());
        }

        return $reservation;
    }

    /**
     * @param array $item
     *
     * @throws RuntimeException
     *
     * @since 1.0
     */
    protected function requireFields($item)
    {
        if (!isset($item['service_id'], $item['date'], $item['time'])) {
            throw new RuntimeException(esc_html__('Unable to make a reservation. Your order misses some of the required data.', 'motopress-appointment'));
        }
    }

    /**
     * @param array $item
     * @return Entities\Service
     *
     * @throws RuntimeException
     *
     * @since 1.0
     */
    protected function parseService($item)
    {
        $serviceId = absint($item['service_id']);
        $service   = mpa_get_service($serviceId);

        if (is_null($service)) {
            // Translators: %d: ID of the post.
            throw new RuntimeException(sprintf(esc_html__('Unable to make a reservation. There is no such service with ID #%d.', 'motopress-appointment'), $serviceId));
        }

        return $service;
    }

    /**
     * @param array $item
     * @return \DateTime
     *
     * @throws RuntimeException
     *
     * @since 1.0
     */
    protected function parseDate($item)
    {
        $date = mpa_parse_date($item['date']);

        if (!$date) {
            throw new RuntimeException(esc_html__('Unable to make a reservation. The date value is broken.', 'motopress-appointment'));
        }

        return $date;
    }

    /**
     * @param array $item
     * @return TimePeriod
     *
     * @throws RuntimeException
     *
     * @since 1.0
     */
    protected function parseTime($item)
    {
        $time = TimePeriod::createFromPeriod($item['time']);

        if (!$time) {
            throw new RuntimeException(esc_html__('Unable to make a reservation. The time value is broken.', 'motopress-appointment'));
        }

        return $time;
    }

    /**
     * @param array $item
     * @param Entities\Service $service
     * @param \DateTime $date
     * @return Entities\Employee
     *
     * @throws RuntimeException
     *
     * @since 1.0
     */
    protected function parseEmployee($item, $service, $date)
    {
        if (isset($item['employee_id'])) {
            // Just find the selected employee
            $employeeId = absint($item['employee_id']);
            $employee = mpa_get_employee($employeeId);

        } else {
            // Search for employee
            $locationId = isset($item['location_id']) ? absint($item['location_id']) : 0;
            $employee = null;

            foreach ($service->employeeIds as $employeeId) {
                // Filter all employees that work at location $locationId for current $date
                if ($locationId != 0) {
                    $schedule = mpapp()->repositories()->schedule()->findByEmployee($employeeId);

                    if (is_null($schedule) || !$schedule->isWorkingAt($locationId, $date)) {
                        continue;
                    }
                }

                $employee = mpa_get_employee($employeeId);

                if (!is_null($employee)) {
                    break; // Found one
                }
            } // For each employee
        }

        if (is_null($employee)) {
            throw new RuntimeException(esc_html__('Unable to make a reservation. No employee found for your request.', 'motopress-appointment'));
        }

        return $employee;
    }

    /**
     * @param array $item
     * @param Entities\Employee $employee
     * @param \DateTime $date
     * @return Entities\Location
     *
     * @throws RuntimeException
     *
     * @since 1.0
     */
    protected function parseLocation($item, $employee, $date)
    {
        if (isset($item['location_id'])) {
            // Just find the selected location
            $locationId = absint($item['location_id']);
            $location = mpa_get_location($locationId);

        } else {
            // Search for location
            $schedule  = mpapp()->repositories()->schedule()->findByEmployee($employee->id);
            $locations = !is_null($schedule) ? $schedule->getLocationIdsForDate($date) : [];
            $location  = !empty($locations) ? mpa_get_location($locations[0]) : null;
        }

        if (is_null($location)) {
            throw new RuntimeException(esc_html__('Unable to make a reservation. No places left for this service.', 'motopress-appointment'));
        }

        return $location;
    }

	/**
	 * @since 1.3.1
	 *
	 * @param array $item
	 *     @param int  $item['capacity']     Optional. 1 by default.
	 *     @param int  $item['bring_people'] Optional. The number of additional
	 *                                       guests. 0 by default.
	 * @param Entities\Service $service
	 * @param Entities\Employee $employee
	 * @return int Valid capacity for the service.
	 */
	protected function parseCapacity($item, $service, $employee)
	{
		$item += [
			'capacity'     => $service->getMinCapacity($employee->id),
			'bring_people' => 0,
		];

		$capacity = $item['capacity'];

		if ($service->allowGuests) {
			$capacity += $item['bring_people'];
		}

		$minCapacity = $service->getMinCapacity($employee->id);
		$maxCapacity = $service->getMaxCapacity($employee->id);

		$capacity = mpa_limit($capacity, $minCapacity, $maxCapacity);

		return $capacity;
	}

    /**
     * @param array $order
     * @return Entities\Customer
     *
     * @since 1.0
     */
    protected function parseCustomer($order)
    {
        $customerInfo = [];

        if (isset($order['customer']) && is_array($order['customer'])) {
            $customerInfo = array_map('sanitize_text_field', $order['customer']);
        }

        return new Entities\Customer(0, $customerInfo);
    }

    /**
     * @param Entities\Booking $booking
     * @return int|false New booking ID or false.
     *
     * @since 1.0
     */
    public function saveBooking($booking)
    {
        $postId = wp_insert_post(
            [
                'post_type'   => PostTypes\BookingPostType::POST_TYPE,
                'post_status' => mpapp()->settings()->getDefaultBookingStatus()
            ]
        );

        if (is_wp_error($postId) || $postId === 0) {
            return false;
        } else {
            $booking->id = $booking->customer->id = $postId;
            $booking->status = mpapp()->settings()->getDefaultBookingStatus();
        }

        // Set title and post name
        wp_update_post(
            [
                'ID'         => $postId,
                'post_title' => 'Booking #' . $postId,
                'post_name'  => 'mpa-booking-' . $postId
            ]
        );

        update_post_meta($postId, '_mpa_total_price',    $booking->totalPrice);
        update_post_meta($postId, '_mpa_customer_name',  $booking->customer->name);
        update_post_meta($postId, '_mpa_customer_email', $booking->customer->email);
        update_post_meta($postId, '_mpa_customer_phone', $booking->customer->phone);

        foreach ($booking->reservations as $reservation) {
            $reservation->bookingId = $postId;

            $childId = wp_insert_post(
                [
                    'post_type'   => PostTypes\ReservationPostType::POST_TYPE,
                    'post_status' => 'publish',
                    'post_parent' => $postId
                ]
            );

            if (is_wp_error($childId) || $childId === 0) {
                continue;
            } else {
                $reservation->id = $childId;
            }

            wp_update_post(
                [
                    'ID'         => $childId,
                    'post_title' => 'Reservation #' . $childId,
                    'post_name'  => 'mpa-reservation-' . $childId
                ]
            );

            update_post_meta($childId, '_mpa_price',        $reservation->price);
            update_post_meta($childId, '_mpa_date',         mpa_format_date($reservation->date, 'internal'));
            update_post_meta($childId, '_mpa_service_time', $reservation->serviceTime->toString('internal'));
            update_post_meta($childId, '_mpa_buffer_time',  $reservation->bufferTime->toString('internal'));
            update_post_meta($childId, '_mpa_service',      $reservation->serviceId);
            update_post_meta($childId, '_mpa_employee',     $reservation->employeeId);
            update_post_meta($childId, '_mpa_location',     $reservation->locationId);
            update_post_meta($childId, '_mpa_capacity',     $reservation->capacity);
        } // For each reservation

        if ($postId !== false) {
            /**
             * Fires after actions "mpa_new_booking_created" and "mpa_booking_{$newStatus}".
             *
             * @see \MotoPress\Appointment\PostTypes\Statuses\BookingStatuses::notifyTransition()
             *
             * @since 1.1.0
             */
            do_action('mpa_booking_placed_by_user', $booking);
        }

        return $postId;
    }
}
