<?php

namespace MotoPress\Appointment\Services;

use MotoPress\Appointment\Entities\Service;
use MotoPress\Appointment\Structures\TimePeriod;
use MotoPress\Appointment\Structures\TimePeriods;
use DateTime;

if (!defined('ABSPATH')) {
	exit;
}

/**
 * Builds time slots for the Appointment Form.
 *
 * @since 1.2.1
 */
class TimeSlotService
{
	/**
	 * @since 1.2.1
	 * @var Service
	 */
	public $service = null;

	/**
	 * @since 1.2.2
	 * @var int[]
	 */
	public $employeeIds = [];

	/**
	 * @since 1.2.2
	 * @var int[]
	 */
	public $locationIds = [];

	/**
	 * @since 1.2.2
	 * @var array [Employee ID => [Location ID => ScheduleService]]
	 */
	public $schedulers = [];

	/**
	 * @since 1.2.1
	 *
	 * @param Service $service
	 * @param array $args Optional.
	 *     @param int|int[] $args['employees'] One or more allowed employee ID.
	 *         Leave empty to allow all employees of the service.
	 *     @param int|int[] $args['locations'] One or more allowed location ID.
	 *         Leave empty to allow all locations.
	 */
	public function __construct($service, $args = [])
	{
		$this->service = $service;

		$employees = !empty($args['employees']) ? (array)$args['employees'] : [];
		$locations = !empty($args['locations']) ? (array)$args['locations'] : [];

		// Don't allow employees that does not perform this service
		if (!empty($employees)) {
			$employees = array_intersect($employees, $service->employeeIds);
		} else {
			$employees = $service->employeeIds;
		}

		// Create ScheduleService for each employee and location
		$this->addEmployees($employees, $locations);
	}

	/**
	 * @since 1.2.1
	 *
	 * @param int[] $employees
	 * @param int|int[] $locations Optional. One or more allowed location ID.
	 *     0 by default (all allowed).
	 */
	public function addEmployees($employees, $locations = 0)
	{
		foreach ($employees as $employeeId) {
			$this->addEmployee($employeeId, $locations);
		}
	}

	/**
	 * @since 1.2.1
	 *
	 * @param int $employeeId
	 * @param int|int[] $locationId Optional. One or more allowed location ID.
	 *     0 by default (all allowed).
	 */
	public function addEmployee($employeeId, $locationId = 0)
	{
		$schedule = mpapp()->repositories()->schedule()->findByEmployee($employeeId);

		if (is_null($schedule)) {
			return;
		}

		$schedulers = ScheduleService::splitScheduleByLocation($schedule, $locationId);

		if (empty($schedulers)) {
			return;
		}

		// Add new schedulers
		foreach ($schedulers as $locationId => $scheduler) {
			$this->schedulers[$employeeId][$locationId] = $scheduler;
		}

		// Merge new IDs
		$this->employeeIds[] = $employeeId;
		$this->locationIds = array_merge($this->locationIds, array_keys($schedulers));

		// Remove duplicate IDs
		$this->employeeIds = mpa_array_unique_reset($this->employeeIds);
		$this->locationIds = mpa_array_unique_reset($this->locationIds);
	}

	/**
	 * @since 1.2.1
	 *
	 * @param DateTime|string $fromDate
	 * @param DateTime|string $toDate
	 */
	public function blockPeriod($fromDate, $toDate)
	{
		$searchArgs = [
			'service_id'  => $this->service->id,
			'employee_id' => $this->employeeIds,
			'location_id' => $this->locationIds,
			'from_date'   => $fromDate,
			'to_date'     => $toDate,
		];

		$reservations = mpapp()->repositories()->reservation()->findAll($searchArgs);

		// Block all periods of reservation
		foreach ($reservations as $reservation) {
			$employeeId = $reservation->employeeId;
			$locationId = $reservation->locationId;

			if (isset($this->schedulers[$employeeId][$locationId])) {
				$scheduler = $this->schedulers[$employeeId][$locationId];
				$scheduler->addReservation($reservation->date, $reservation->bufferTime);
			}
		}
	}

	/**
	 * @since 1.4.0
	 *
	 * @param array $cartItems Array of [service_id, employee_id, location_id,
	 *		date, time].
	 */
	public function blockCartItems($cartItems)
	{
		foreach ($cartItems as $cartItem) {
			if ($cartItem['service_id'] !== $this->service->id) {
				continue;
			}

			$employeeId = $cartItem['employee_id'];
			$locationId = $cartItem['location_id'];

			// Find proper scheduler and block occupied date/time
			if (isset($this->schedulers[$employeeId][$locationId])) {
				$scheduler  = $this->schedulers[$employeeId][$locationId];
				$timePeriod = mpa_add_buffer_time($cartItem['time'], $this->service);

				$scheduler->addReservation($cartItem['date'], $timePeriod);
			}
		}
	}

	/**
	 * @since 1.2.1
	 *
	 * @param DateTime|string $fromDate
	 * @param DateTime|string $toDate
	 * @param bool $sinceToday Optional. Limit the min date to the current date & time
	 *     + time before booking. True by default.
	 * @return array ['Y-m-d' => [Time period string => Array of [Employee ID, Location ID]]]
	 */
	public function getTimeSlotsForEntities($fromDate, $toDate, $sinceToday = true)
	{
		$fromDate = mpa_parse_date($fromDate);
		$toDate   = mpa_parse_date($toDate);

		if ($fromDate === false || $toDate === false) {
			return [];
		}

		// Find available time slots for each employee/location
		$slots = [];

		// For each employee
		foreach ($this->schedulers as $employeeId => $schedulers) {
			$slotsArgs = [
				'since_today' => $sinceToday,
				'employee_id' => $employeeId,
			];

			// For each schedule
			foreach ($schedulers as $locationId => $scheduler) {
				$workingDays = $scheduler->getWorkingHoursForPeriod($fromDate, $toDate);

				// For each day
				foreach ($workingDays as $dateStr => $workingHours) {
					$timeSlots = $this->getTimeSlotsForHours($workingHours, $slotsArgs);

					// For each time slot
					foreach (array_keys($timeSlots) as $timeStr) {
						$slots[$dateStr][$timeStr][] = [$employeeId, $locationId];
					} // For each time slot
				} // For each day
			} // For each schedule
		} // For each employee

		// Sort days in ascending order
		ksort($slots);

		// Sort time periods in ascending order
		foreach ($slots as &$daySlots) {
			ksort($daySlots);
		}

		unset($daySlots);

		return $slots;
	}

	/**
	 * @since 1.2.1
	 *
	 * @param DateTime|string $fromDate
	 * @param DateTime|string $toDate
	 * @param array $args Optional.
	 *     @param bool $args['skip_empty']  Skip days without available slots. True
	 *                                      by default.
	 *     @param bool $args['since_today'] Limit the min date to the current date
	 *                                      & time + time before booking. True by
	 *                                      default.
	 * @return array ['Y-m-d' => [Time period string => Time period]]
	 *
	 * @todo Return empty days instead of [] when $skipEmpty=false and $fromDate=false or $toDate=false.
	 */
	public function getAvailableTimeSlotsForPeriod($fromDate, $toDate, $args = [])
	{
		$fromDate = mpa_parse_date($fromDate);
		$toDate   = mpa_parse_date($toDate);

		if ($fromDate === false || $toDate === false) {
			return [];
		}

		// Add defaults
		$args += [
			'skip_empty'  => true,
			'since_today' => true,
		];

		/**
		 * @var bool $skip_empty
		 * @var bool $since_today
		 */
		extract($args);

		// Find available time slots
		$slots = [];

		for ($date = clone $fromDate; $date <= $toDate; $date->modify('+1 day')) {
			$timeSlots = $this->getAvailableTimeSlotsForDate($date, $since_today);

			if (!empty($timeSlots) || !$skip_empty) {
				$dateStr = mpa_format_date($date, 'internal');
				$slots[$dateStr] = $timeSlots;
			}
		}

		return $slots;
	}

	/**
	 * @since 1.2.1
	 *
	 * @param DateTime $date
	 * @param bool $sinceToday Optional. Limit the min date to the current date
	 *     & time + time before booking. True by default.
	 * @return TimePeriod[] [Time period string => Time period]
	 */
	public function getAvailableTimeSlotsForDate($date, $sinceToday = true)
	{
		$timeSlots = [];

		foreach ($this->schedulers as $employeeId => $schedulers) {
			$slotsArgs = [
				'since_today' => $sinceToday,
				'employee_id' => $employeeId,
			];

			foreach ($schedulers as $scheduler) {
				$workingHours = $scheduler->getWorkingHoursForDate($date);
				$workingSlots = $this->getTimeSlotsForHours($workingHours, $slotsArgs);

				$timeSlots += $workingSlots;
			}
		}

		return $timeSlots;
	}

	/**
	 * @since 1.2.1
	 *
	 * @param DateTime $date
	 * @param bool $sinceToday Optional. Limit the min date to the current date
	 *     & time + time before booking. True by default.
	 * @return bool
	 */
	public function isAvailableDate($date, $sinceToday = true)
	{
		$timeSlots = $this->getAvailableTimeSlotsForDate($date, $sinceToday);

		return !empty($timeSlots);
	}

	/**
	 * @since 1.2.1
	 *
	 * @param TimePeriod|TimePeriod[]|TimePeriods $workingHours
	 * @param array $args Optional.
	 *     @param bool $args['since_today'] Limit the min date to the current
	 *                                      date & time + time before booking.
	 *                                      True by default.
	 *     @param int  $args['employee_id'] ID for variations. 0 by default (no
	 *                                      variation).
	 * @return TimePeriod[] [Time period string => Time period]
	 */
	public function getTimeSlotsForHours($workingHours, $args = [])
	{
		// Add defaults
		$args += [
			'since_today' => true,
			'employee_id' => 0,
		];

		// Prepare args for mpa_time_slots()
		$slotsArgs = [
			// Use duration from available variations
			'duration'      => $this->service->getDuration($args['employee_id']),
			'buffer_before' => $this->service->bufferTimeBefore,
			'buffer_after'  => $this->service->bufferTimeAfter,
		];

		if ($args['since_today']) {
			$minTime = new DateTime();
			$minTime->modify("+{$this->service->timeBeforeBooking} minutes");

			$slotsArgs['min_time'] = $minTime;
		}

		return mpa_time_slots($workingHours, $slotsArgs);
	}
}
